@@ -319,10 +319,10 @@ static int ai_goldequiv_clause(struct player *pplayer,
          * - AI losses tech due to high upkeep. 
          * FIXME: Is there a better way for this? */
         struct player_research *research = player_research_get(pplayer);
-        int limit = MAX(1, research->tech_upkeep
+        int limit = MAX(1, player_tech_upkeep(pplayer)
                            / research->techs_researched);
 
-        if (pplayer->bulbs_last_turn < limit) {
+        if (pplayer->server.bulbs_last_turn < limit) {
           worth /= 2;
         }
       }
@@ -439,7 +439,7 @@ static int ai_goldequiv_clause(struct player *pplayer,
     if (give) {
       worth -= pclause->value;
     } else {
-      worth += pclause->value;
+      worth += pclause->value * (100 - game.server.diplcost) / 100;
     }
     break;
 
