@@ -132,7 +132,7 @@ const char *ai_choice_rule_name(const struct adv_choice *choice)
 int military_amortize(struct player *pplayer, struct city *pcity,
                       int value, int delay, int build_cost)
 {
-  struct adv_data *ai = adv_data_get(pplayer);
+  struct adv_data *ai = adv_data_get(pplayer, NULL);
   int city_output = (pcity ? pcity->surplus[O_SHIELD] : 1);
   int output = MAX(city_output, ai->stats.average_production);
   int build_time = build_cost / MAX(output, 1);
@@ -1005,11 +1005,11 @@ int stack_cost(struct unit *pattacker, struct unit *pdefender)
   if (is_stack_vulnerable(ptile)) {
     /* lotsa people die */
     unit_list_iterate(ptile->units, aunit) {
-      if (can_unit_attack_unit_at_tile(pattacker, aunit, ptile)) {
+      if (unit_attack_unit_at_tile_result(pattacker, aunit, ptile) == ATT_OK) {
         victim_cost += unit_build_shield_cost(aunit);
       }
     } unit_list_iterate_end;
-  } else if (can_unit_attack_unit_at_tile(pattacker, pdefender, ptile)) {
+  } else if (unit_attack_unit_at_tile_result(pattacker, pdefender, ptile) == ATT_OK) {
     /* Only one unit dies if attack is successful */
     victim_cost = unit_build_shield_cost(pdefender);
   }
