@@ -347,12 +347,14 @@ void nuclear_winter_scaled(int *chance, int *rate, int max)
 struct sprite *client_research_sprite(void)
 {
   if (NULL != client.conn.playing && can_client_change_view()) {
+    const struct player_research *presearch =
+        player_research_get(client_player());
     int index = 0;
 
-    if (A_UNSET != player_research_get(client.conn.playing)->researching) {
-      index = (NUM_TILES_PROGRESS
-	       * player_research_get(client.conn.playing)->bulbs_researched)
-	/ (total_bulbs_required(client.conn.playing) + 1);
+    if (is_future_tech(presearch->researching)
+        || NULL != valid_advance_by_number(presearch->researching)) {
+      index = (NUM_TILES_PROGRESS * presearch->bulbs_researched
+               / (presearch->client.researching_cost + 1));
     }
 
     /* This clipping can be necessary since we can end up with excess
@@ -1310,16 +1312,6 @@ void client_player_init(struct player *pplayer)
   } vision_layer_iterate_end;
 }
 
-/***************************************************************
- Destroy a player on the client side.
-***************************************************************/
-void client_player_destroy(struct player *pplayer)
-{
-  vision_layer_iterate(v) {
-    dbv_free(&pplayer->client.tile_vision[v]);
-  } vision_layer_iterate_end;
-}
-
 /***************************************************************
   Reset the private maps of all players.
 ***************************************************************/
