@@ -77,7 +77,7 @@ struct goto_map {
 /* Iterate over goto maps, assumes no dead units. */
 #define goto_map_unit_iterate(gotolist, pgoto, punit)			\
   goto_map_list_iterate(gotolist, pgoto) {				\
-    struct unit *punit = pgoto->focus;
+    struct unit *punit = goto_map_unit(pgoto);
 
 #define goto_map_unit_iterate_end					\
   } goto_map_list_iterate_end;
@@ -124,6 +124,19 @@ static void goto_map_free(struct goto_map *goto_map)
   free(goto_map);
 }
 
+/****************************************************************************
+  Returns the unit associated with the goto map.
+****************************************************************************/
+static struct unit *goto_map_unit(const struct goto_map *goto_map)
+{
+  struct unit *punit = goto_map->focus;
+
+  fc_assert(punit != NULL);
+  fc_assert(unit_is_in_focus(punit));
+  fc_assert(punit == player_unit_by_number(client_player(), punit->id));
+  return punit;
+}
+
 /********************************************************************** 
   Called only by handle_map_info() in client/packhand.c.
 ***********************************************************************/
@@ -297,11 +310,7 @@ static void add_part(struct goto_map *goto_map)
 {
   struct part *p;
   struct pf_parameter parameter = goto_map->template;
-  struct unit *punit = goto_map->focus;
-
-  if (!punit) {
-    return;
-  }
+  struct unit *punit = goto_map_unit(goto_map);
 
   goto_map->num_parts++;
   goto_map->parts =
@@ -354,6 +363,7 @@ static void remove_last_part(struct goto_map *goto_map)
 bool goto_add_waypoint(void)
 {
   struct tile *ptile_start = NULL;
+  bool duplicate_of_last = TRUE;
 
   fc_assert_ret_val(goto_is_active(), FALSE);
   if (NULL == goto_destination) {
@@ -374,7 +384,13 @@ bool goto_add_waypoint(void)
       /* Scattered group (not all in same location). */
       return FALSE;
     }
+    if (last_part->start_tile != last_part->end_tile) {
+      duplicate_of_last = FALSE;
+    }
   } goto_map_list_iterate_end;
+  if (duplicate_of_last) {
+    return FALSE;
+  }
 
   goto_map_list_iterate(goto_maps, goto_map) {
     add_part(goto_map);
@@ -731,21 +747,51 @@ no_fights_or_unknown_goto(const struct tile *ptile,
   return no_fights_or_unknown(ptile, known, p);
 }
 
-/********************************************************************** 
+/****************************************************************************
+  Fill the PF parameter with the correct client-goto values.
+  See also goto_fill_parameter_full().
+****************************************************************************/
+static void goto_fill_parameter_base(struct pf_parameter *parameter,
+                                     struct unit *punit)
+{
+  pft_fill_unit_parameter(parameter, punit);
+  /* Overwrite 'omniscience' in case client player is under AI control. */
+  parameter->omniscience = FALSE;
+
+  fc_assert(parameter->get_EC == NULL);
+  fc_assert(parameter->get_TB == NULL);
+  fc_assert(parameter->get_MC != NULL);
+  fc_assert(parameter->start_tile == unit_tile(punit));
+
+  parameter->get_EC = get_EC;
+  if (is_attack_unit(punit) || is_diplomat_unit(punit)) {
+    parameter->get_TB = get_TB_aggr;
+  } else if (unit_has_type_flag(punit, F_TRADE_ROUTE)
+             || unit_has_type_flag(punit, F_HELP_WONDER)) {
+    parameter->get_TB = get_TB_caravan;
+  } else {
+    parameter->get_TB = no_fights_or_unknown_goto;
+  }
+}
+
+/****************************************************************************
   Fill the PF parameter with the correct client-goto values.
   The storage behind "connect_speed" must remain valid for the lifetime
   of the pf_map.
-***********************************************************************/
-static void fill_client_goto_parameter(struct unit *punit,
-				       struct pf_parameter *parameter,
-				       int *initial_mp,
-				       int *connect_speed)
+
+  Note that you must call this function only if the 'hover_state' is set,
+  and we are in goto mode (usually, this function is only called from
+  enter_goto_state()).
+
+  See also goto_fill_parameter_base().
+****************************************************************************/
+static void goto_fill_parameter_full(struct pf_parameter *parameter,
+                                     struct unit *punit,
+                                     int *initial_mp, int *connect_speed)
 {
-  pft_fill_unit_parameter(parameter, punit);
-  fc_assert(parameter->get_EC == NULL);
-  parameter->get_EC = get_EC;
-  fc_assert(parameter->get_TB == NULL);
-  fc_assert(parameter->get_MC != NULL);
+  goto_fill_parameter_base(parameter, punit);
+
+  fc_assert_ret(goto_is_active());
 
   switch (hover_state) {
   case HOVER_CONNECT:
@@ -781,28 +827,25 @@ static void fill_client_goto_parameter(struct unit *punit,
     }
     break;
   case HOVER_NUKE:
-    parameter->get_moves_left_req = NULL; /* nuclear safety? pwah! */
-    /* FALLTHRU */
-  default:
+    /* We only want targets reachable immediatly... */
+    parameter->move_rate = 0;
+    /* ...then we don't need to deal with dangers or refuel points. */
+    parameter->is_pos_dangerous = NULL;
+    parameter->get_moves_left_req = NULL;
+    *initial_mp = -parameter->moves_left_initially;
+    break;
+  case HOVER_GOTO:
+  case HOVER_PATROL:
     *initial_mp = parameter->move_rate - parameter->moves_left_initially;
     break;
+  case HOVER_NONE:
+  case HOVER_PARADROP:
+    fc_assert_msg(hover_state != HOVER_NONE, "Goto with HOVER_NONE?");
+    fc_assert_msg(hover_state != HOVER_PARADROP,
+                  "Goto with HOVER_PARADROP?");
+    *initial_mp = 0;
+    break;
   };
-
-  if (is_attack_unit(punit) || is_diplomat_unit(punit)) {
-    parameter->get_TB = get_TB_aggr;
-  } else if (unit_has_type_flag(punit, F_TRADE_ROUTE)
-	     || unit_has_type_flag(punit, F_HELP_WONDER)) {
-    parameter->get_TB = get_TB_caravan;
-  } else {
-    parameter->get_TB = no_fights_or_unknown_goto;
-  }
-
-  /* Note that in connect mode the "time" does not correspond to any actual
-   * move rate. */
-  parameter->start_tile = unit_tile(punit);
-
-  /* Omniscience is always FALSE in the client */
-  parameter->omniscience = FALSE;
 }
 
 /********************************************************************** 
@@ -820,14 +863,12 @@ void enter_goto_state(struct unit_list *punits)
     struct goto_map *goto_map = goto_map_new();
 
     goto_map->focus = punit;
-
-    fill_client_goto_parameter(punit, &goto_map->template,
-                               &goto_map->initial_mp,
-                               &goto_map->connect_speed);
-
-    add_part(goto_map);
-
     goto_map_list_append(goto_maps, goto_map);
+
+    goto_fill_parameter_full(&goto_map->template, punit,
+                             &goto_map->initial_mp,
+                             &goto_map->connect_speed);
+    add_part(goto_map);
   } unit_list_iterate_end;
   goto_warned = FALSE;
 }
@@ -906,17 +947,18 @@ bool goto_get_turns(int *min, int *max)
       mp += goto_map->parts[i].mp;
     }
 
-    mp = MAX(0, mp);
-
-    if (goto_map->template.move_rate == 0) {
-      /* Immobile unit can never reach destination. */
-      return FALSE;
+    if (goto_map->template.move_rate > 0) {
+      /* Round down -- if we can get there this turn with MP left, report 0,
+       * if we get there with 0 MP, report 1 */
+      turns = MAX(mp, 0) / goto_map->template.move_rate;
+    } else if (goto_map->template.moves_left_initially > 0) {
+      turns = (mp == 0);
+    } else {
+      /* Immobile unit: it is current position, or it would have returned
+       * when testing 'goto_destination'. */
+      turns = 0;
     }
 
-    /* Round down -- if we can get there this turn with MP left, report 0,
-     * if we get there with 0 MP, report 1 */
-    turns = mp / goto_map->template.move_rate;
-
     if (min) {
       *min = MIN(*min, turns);
     }
@@ -987,6 +1029,15 @@ static void send_path_orders(struct unit *punit, struct pf_path *path,
   int i;
   struct tile *old_tile;
 
+  fc_assert_ret(path != NULL);
+  fc_assert_ret_msg(unit_tile(punit) == path->positions[0].tile,
+                    "Unit %d has moved without goto cancelation.",
+                    punit->id);
+
+  if (path->length == 1 && final_order == NULL) {
+    return; /* No path at all, no need to spam the server. */
+  }
+
   memset(&p, 0, sizeof(p));
   p.unit_id = punit->id;
   p.src_tile = tile_index(unit_tile(punit));
@@ -1054,12 +1105,11 @@ void send_goto_path(struct unit *punit, struct pf_path *path,
 ****************************************************************************/
 bool send_goto_tile(struct unit *punit, struct tile *ptile)
 {
-  int dummy1, dummy2;
   struct pf_parameter parameter;
   struct pf_map *pfm;
   struct pf_path *path;
 
-  fill_client_goto_parameter(punit, &parameter, &dummy1, &dummy2);
+  goto_fill_parameter_base(&parameter, punit);
   pfm = pf_map_new(&parameter);
   path = pf_map_path(pfm, ptile);
   pf_map_destroy(pfm);
@@ -1105,9 +1155,9 @@ void send_patrol_route(void)
     }
 
     for (i = 0; i < goto_map->num_parts; i++) {
-      path = pft_concat(path, goto_map->parts[i].path);
+      path = pf_path_concat(path, goto_map->parts[i].path);
     }
-    path = pft_concat(path, return_path);
+    path = pf_path_concat(path, return_path);
 
     pf_map_destroy(pfm);
     pf_path_destroy(return_path);
@@ -1140,7 +1190,7 @@ void send_connect_route(enum unit_activity activity)
     memset(&p, 0, sizeof(p));
 
     for (i = 0; i < goto_map->num_parts; i++) {
-      path = pft_concat(path, goto_map->parts[i].path);
+      path = pf_path_concat(path, goto_map->parts[i].path);
     }
 
     p.unit_id = punit->id;
@@ -1229,7 +1279,7 @@ void send_goto_route(void)
     }
 
     for (i = 0; i < goto_map->num_parts; i++) {
-      path = pft_concat(path, goto_map->parts[i].path);
+      path = pf_path_concat(path, goto_map->parts[i].path);
     }
 
     clear_unit_orders(punit);
@@ -1260,7 +1310,6 @@ void send_goto_route(void)
 ***************************************************************************/
 struct pf_path *path_to_nearest_allied_city(struct unit *punit)
 {
-  int dummy1, dummy2;
   struct pf_parameter parameter;
   struct pf_map *pfm;
   struct pf_path *path = NULL;
@@ -1270,7 +1319,7 @@ struct pf_path *path_to_nearest_allied_city(struct unit *punit)
     return NULL;
   }
 
-  fill_client_goto_parameter(punit, &parameter, &dummy1, &dummy2);
+  goto_fill_parameter_base(&parameter, punit);
   pfm = pf_map_new(&parameter);
 
   pf_map_tiles_iterate(pfm, ptile, FALSE) {
