@@ -111,7 +111,8 @@ void popup_caravan_dialog(struct unit *punit,
   
   get_help_build_wonder_button_label(wonder, sizeof(wonder), &can_wonder);
   
-  can_trade = can_cities_trade(phomecity, pdestcity);
+  can_trade = (unit_has_type_flag(punit, F_TRADE_ROUTE)
+               && can_cities_trade(phomecity, pdestcity));
   can_establish = can_trade
   		  && can_establish_trade_route(phomecity, pdestcity);
 
