@@ -23,6 +23,7 @@
 
 /* common */
 #include "game.h"
+#include "movement.h"
 #include "unit.h"
 #include "unitlist.h"
 
@@ -39,31 +40,60 @@
 #include "mapview.h"
 #include "packhand.h"
 
-/* client/gui-gtk-3.0 */
+/* client/gui-gtk-2.0 */
 #include "citydlg.h"
 #include "dialogs.h"
 #include "wldlg.h"
 
 static GtkWidget *diplomat_dialog;
 static int diplomat_id;
-static int diplomat_target_id;
 
 static GtkWidget  *spy_tech_shell;
-static int         steal_advance;
 
 static GtkWidget  *spy_sabotage_shell;
-static int         sabotage_improvement;
+
+/* A structure to hold parameters for actions inside the GUI in stead of
+ * storing the needed data in a global variable. */
+struct action_data {
+  int actor_unit_id;
+  int target_city_id;
+  int target_unit_id;
+  int value;
+};
 
 /****************************************************************
-  User responded to bribe dialog
+  Create a new action data structure that can be stored in the
+  dialogs.
 *****************************************************************/
-static void bribe_response(GtkWidget *w, gint response)
+static struct action_data *act_data(int actor_unit_id,
+                                    int target_city_id,
+                                    int target_unit_id,
+                                    int value)
 {
+  struct action_data *data = fc_malloc(sizeof(*data));
+
+  data->actor_unit_id = actor_unit_id;
+  data->target_city_id = target_city_id;
+  data->target_unit_id = target_unit_id;
+  data->value = value;
+
+  return data;
+}
+
+/**********************************************************************
+  User responded to bribe dialog
+**********************************************************************/
+static void bribe_response(GtkWidget *w, gint response, gpointer data)
+{
+  struct action_data *args = (struct action_data *)data;
+
   if (response == GTK_RESPONSE_YES) {
-    request_diplomat_action(DIPLOMAT_BRIBE, diplomat_id,
-			    diplomat_target_id, 0);
+    request_diplomat_action(DIPLOMAT_BRIBE, args->actor_unit_id,
+                            args->target_unit_id, 0);
   }
+
   gtk_widget_destroy(w);
+  free(args);
 }
 
 /****************************************************************
@@ -71,18 +101,22 @@ static void bribe_response(GtkWidget *w, gint response)
 *****************************************************************/
 static void diplomat_bribe_callback(GtkWidget *w, gpointer data)
 {
-  if (NULL != game_unit_by_number(diplomat_id)
-      && NULL != game_unit_by_number(diplomat_target_id)) {
-    request_diplomat_answer(DIPLOMAT_BRIBE, diplomat_id,
-                            diplomat_target_id, 0);
+  struct action_data *args = (struct action_data *)data;
+
+  if (NULL != game_unit_by_number(args->actor_unit_id)
+      && NULL != game_unit_by_number(args->target_unit_id)) {
+    request_diplomat_answer(DIPLOMAT_BRIBE, args->actor_unit_id,
+                            args->target_unit_id, 0);
   }
+
   gtk_widget_destroy(diplomat_dialog);
+  free(args);
 }
 
-/****************************************************************
+/*************************************************************************
   Popup unit bribe dialog
-*****************************************************************/
-void popup_bribe_dialog(struct unit *punit, int cost)
+**************************************************************************/
+void popup_bribe_dialog(struct unit *actor, struct unit *punit, int cost)
 {
   GtkWidget *shell;
   char buf[1024];
@@ -117,7 +151,8 @@ void popup_bribe_dialog(struct unit *punit, int cost)
   }
   gtk_window_present(GTK_WINDOW(shell));
   
-  g_signal_connect(shell, "response", G_CALLBACK(bribe_response), NULL);
+  g_signal_connect(shell, "response", G_CALLBACK(bribe_response),
+                   act_data(actor->id, 0, punit->id, cost));
 }
 
 /****************************************************************
@@ -125,12 +160,16 @@ void popup_bribe_dialog(struct unit *punit, int cost)
 *****************************************************************/
 static void diplomat_sabotage_callback(GtkWidget *w, gpointer data)
 {
-  if (NULL != game_unit_by_number(diplomat_id)
-      && NULL != game_city_by_number(diplomat_target_id)) {
-    request_diplomat_action(DIPLOMAT_SABOTAGE, diplomat_id,
-                            diplomat_target_id, B_LAST + 1);
+  struct action_data *args = (struct action_data *)data;
+
+  if (NULL != game_unit_by_number(args->actor_unit_id)
+      && NULL != game_city_by_number(args->target_city_id)) {
+    request_diplomat_action(DIPLOMAT_SABOTAGE, args->actor_unit_id,
+                            args->target_city_id, B_LAST + 1);
   }
+
   gtk_widget_destroy(diplomat_dialog);
+  free(args);
 }
 
 /****************************************************************
@@ -138,12 +177,16 @@ static void diplomat_sabotage_callback(GtkWidget *w, gpointer data)
 *****************************************************************/
 static void diplomat_investigate_callback(GtkWidget *w, gpointer data)
 {
-  if (NULL != game_city_by_number(diplomat_target_id)
-      && NULL != game_unit_by_number(diplomat_id)) {
-    request_diplomat_action(DIPLOMAT_INVESTIGATE, diplomat_id,
-                            diplomat_target_id, 0);
+  struct action_data *args = (struct action_data *)data;
+
+  if (NULL != game_city_by_number(args->target_city_id)
+      && NULL != game_unit_by_number(args->actor_unit_id)) {
+    request_diplomat_action(DIPLOMAT_INVESTIGATE, args->actor_unit_id,
+                            args->target_city_id, 0);
   }
+
   gtk_widget_destroy(diplomat_dialog);
+  free(args);
 }
 
 /****************************************************************
@@ -151,9 +194,13 @@ static void diplomat_investigate_callback(GtkWidget *w, gpointer data)
 *****************************************************************/
 static void spy_sabotage_unit_callback(GtkWidget *w, gpointer data)
 {
-  request_diplomat_action(SPY_SABOTAGE_UNIT, diplomat_id,
-			  diplomat_target_id, 0);
+  struct action_data *args = (struct action_data *)data;
+
+  request_diplomat_action(SPY_SABOTAGE_UNIT, args->actor_unit_id,
+                          args->target_unit_id, 0);
+
   gtk_widget_destroy(diplomat_dialog);
+  free(args);
 }
 
 /****************************************************************
@@ -161,12 +208,16 @@ static void spy_sabotage_unit_callback(GtkWidget *w, gpointer data)
 *****************************************************************/
 static void diplomat_embassy_callback(GtkWidget *w, gpointer data)
 {
-  if (NULL != game_unit_by_number(diplomat_id)
-      && NULL != game_city_by_number(diplomat_target_id)) {
-    request_diplomat_action(DIPLOMAT_EMBASSY, diplomat_id,
-                            diplomat_target_id, 0);
+  struct action_data *args = (struct action_data *)data;
+
+  if (NULL != game_unit_by_number(args->actor_unit_id)
+      && NULL != game_city_by_number(args->target_city_id)) {
+    request_diplomat_action(DIPLOMAT_EMBASSY, args->actor_unit_id,
+                            args->target_city_id, 0);
   }
+
   gtk_widget_destroy(diplomat_dialog);
+  free(args);
 }
 
 /****************************************************************
@@ -174,11 +225,16 @@ static void diplomat_embassy_callback(GtkWidget *w, gpointer data)
 *****************************************************************/
 static void spy_poison_callback(GtkWidget *w, gpointer data)
 {
-  if (NULL != game_unit_by_number(diplomat_id)
-      && NULL != game_city_by_number(diplomat_target_id)) {
-    request_diplomat_action(SPY_POISON, diplomat_id, diplomat_target_id, 0);
+  struct action_data *args = (struct action_data *)data;
+
+  if (NULL != game_unit_by_number(args->actor_unit_id)
+      && NULL != game_city_by_number(args->target_city_id)) {
+    request_diplomat_action(SPY_POISON, args->actor_unit_id,
+                            args->target_city_id, 0);
   }
+
   gtk_widget_destroy(diplomat_dialog);
+  free(args);
 }
 
 /****************************************************************
@@ -186,45 +242,57 @@ static void spy_poison_callback(GtkWidget *w, gpointer data)
 *****************************************************************/
 static void diplomat_steal_callback(GtkWidget *w, gpointer data)
 {
-  if (NULL != game_unit_by_number(diplomat_id)
-      && NULL != game_city_by_number(diplomat_target_id)) {
-    request_diplomat_action(DIPLOMAT_STEAL, diplomat_id,
-                            diplomat_target_id, A_UNSET);
+  struct action_data *args = (struct action_data *)data;
+
+  if (NULL != game_unit_by_number(args->actor_unit_id)
+      && NULL != game_city_by_number(args->target_city_id)) {
+    request_diplomat_action(DIPLOMAT_STEAL, args->actor_unit_id,
+                            args->target_city_id, A_UNSET);
   }
+
   gtk_widget_destroy(diplomat_dialog);
+  free(args);
 }
 
 /****************************************************************
   User responded to steal advances dialog
 *****************************************************************/
-static void spy_advances_response(GtkWidget *w, gint response, gpointer data)
+static void spy_advances_response(GtkWidget *w, gint response,
+                                  gpointer data)
 {
-  if (response == GTK_RESPONSE_ACCEPT && steal_advance > 0) {
-    if (NULL != game_unit_by_number(diplomat_id)
-        && NULL != game_city_by_number(diplomat_target_id)) {
-      request_diplomat_action(DIPLOMAT_STEAL, diplomat_id,
-                              diplomat_target_id, steal_advance);
+  struct action_data *args = (struct action_data *)data;
+
+  if (response == GTK_RESPONSE_ACCEPT && args->value > 0) {
+    if (NULL != game_unit_by_number(args->actor_unit_id)
+        && NULL != game_city_by_number(args->target_city_id)) {
+      request_diplomat_action(DIPLOMAT_STEAL, args->actor_unit_id,
+                              args->target_city_id, args->value);
     }
   }
+
   gtk_widget_destroy(spy_tech_shell);
   spy_tech_shell = NULL;
+  free(data);
 }
 
 /****************************************************************
   User selected entry in steal advances dialog
 *****************************************************************/
-static void spy_advances_callback(GtkTreeSelection *select, gpointer data)
+static void spy_advances_callback(GtkTreeSelection *select,
+                                  gpointer data)
 {
+  struct action_data *args = (struct action_data *)data;
+
   GtkTreeModel *model;
   GtkTreeIter it;
 
   if (gtk_tree_selection_get_selected(select, &model, &it)) {
-    gtk_tree_model_get(model, &it, 1, &steal_advance, -1);
+    gtk_tree_model_get(model, &it, 1, &(args->value), -1);
     
     gtk_dialog_set_response_sensitive(GTK_DIALOG(spy_tech_shell),
       GTK_RESPONSE_ACCEPT, TRUE);
   } else {
-    steal_advance = 0;
+    args->value = 0;
 	  
     gtk_dialog_set_response_sensitive(GTK_DIALOG(spy_tech_shell),
       GTK_RESPONSE_ACCEPT, FALSE);
@@ -235,7 +303,8 @@ static void spy_advances_callback(GtkTreeSelection *select, gpointer data)
   Create spy's tech stealing dialog
 *****************************************************************/
 static void create_advances_list(struct player *pplayer,
-				 struct player *pvictim)
+				 struct player *pvictim,
+				 struct action_data *args)
 {  
   GtkWidget *sw, *label, *vbox, *view;
   GtkListStore *store;
@@ -257,19 +326,14 @@ static void create_advances_list(struct player *pplayer,
 				  GTK_RESPONSE_ACCEPT);
 
   label = gtk_frame_new(_("Select Advance to Steal"));
-  gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area(GTK_DIALOG(spy_tech_shell))), label);
+  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(spy_tech_shell)->vbox), label);
 
-  vbox = gtk_grid_new();
-  gtk_orientable_set_orientation(GTK_ORIENTABLE(vbox),
-                                 GTK_ORIENTATION_VERTICAL);
-  gtk_grid_set_row_spacing(GTK_GRID(vbox), 6);
+  vbox = gtk_vbox_new(FALSE, 6);
   gtk_container_add(GTK_CONTAINER(label), vbox);
       
   store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
 
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
-  gtk_widget_set_hexpand(view, TRUE);
-  gtk_widget_set_vexpand(view, TRUE);
   g_object_unref(store);
   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
 
@@ -325,7 +389,8 @@ static void create_advances_list(struct player *pplayer,
       struct astring str = ASTRING_INIT;
       /* TRANS: %s is a unit name, e.g., Spy */
       astr_set(&str, _("At %s's Discretion"),
-               unit_name_translation(game_unit_by_number(diplomat_id)));
+               unit_name_translation(game_unit_by_number(
+                                         args->actor_unit_id)));
       g_value_set_string(&value, astr_str(&str));
       astr_free(&str);
     }
@@ -337,14 +402,14 @@ static void create_advances_list(struct player *pplayer,
   gtk_dialog_set_response_sensitive(GTK_DIALOG(spy_tech_shell),
     GTK_RESPONSE_ACCEPT, FALSE);
   
-  gtk_widget_show_all(gtk_dialog_get_content_area(GTK_DIALOG(spy_tech_shell)));
+  gtk_widget_show_all(GTK_DIALOG(spy_tech_shell)->vbox);
 
   g_signal_connect(gtk_tree_view_get_selection(GTK_TREE_VIEW(view)), "changed",
-		   G_CALLBACK(spy_advances_callback), NULL);
+                   G_CALLBACK(spy_advances_callback), args);
   g_signal_connect(spy_tech_shell, "response",
-		   G_CALLBACK(spy_advances_response), NULL);
+                   G_CALLBACK(spy_advances_response), args);
   
-  steal_advance = 0;
+  args->value = 0;
 
   gtk_tree_view_focus(GTK_TREE_VIEW(view));
 }
@@ -354,15 +419,21 @@ static void create_advances_list(struct player *pplayer,
 *****************************************************************/
 static void spy_improvements_response(GtkWidget *w, gint response, gpointer data)
 {
-  if (response == GTK_RESPONSE_ACCEPT && sabotage_improvement > -2) {
-    if (NULL != game_unit_by_number(diplomat_id)
-        && NULL != game_city_by_number(diplomat_target_id)) {
-      request_diplomat_action(DIPLOMAT_SABOTAGE, diplomat_id,
-                              diplomat_target_id, sabotage_improvement + 1);
+  struct action_data *args = (struct action_data *)data;
+
+  if (response == GTK_RESPONSE_ACCEPT && args->value > -2) {
+    if (NULL != game_unit_by_number(args->actor_unit_id)
+        && NULL != game_city_by_number(args->target_city_id)) {
+      request_diplomat_action(DIPLOMAT_SABOTAGE,
+                              args->actor_unit_id,
+                              args->target_city_id,
+                              args->value + 1);
     }
   }
+
   gtk_widget_destroy(spy_sabotage_shell);
   spy_sabotage_shell = NULL;
+  free(args);
 }
 
 /****************************************************************
@@ -370,16 +441,18 @@ static void spy_improvements_response(GtkWidget *w, gint response, gpointer data
 *****************************************************************/
 static void spy_improvements_callback(GtkTreeSelection *select, gpointer data)
 {
+  struct action_data *args = (struct action_data *)data;
+
   GtkTreeModel *model;
   GtkTreeIter it;
 
   if (gtk_tree_selection_get_selected(select, &model, &it)) {
-    gtk_tree_model_get(model, &it, 1, &sabotage_improvement, -1);
+    gtk_tree_model_get(model, &it, 1, &(args->value), -1);
     
     gtk_dialog_set_response_sensitive(GTK_DIALOG(spy_sabotage_shell),
       GTK_RESPONSE_ACCEPT, TRUE);
   } else {
-    sabotage_improvement = -2;
+    args->value = -2;
 	  
     gtk_dialog_set_response_sensitive(GTK_DIALOG(spy_sabotage_shell),
       GTK_RESPONSE_ACCEPT, FALSE);
@@ -390,7 +463,8 @@ static void spy_improvements_callback(GtkTreeSelection *select, gpointer data)
   Creates spy's building sabotaging dialog
 *****************************************************************/
 static void create_improvements_list(struct player *pplayer,
-				     struct city *pcity)
+				     struct city *pcity,
+				     struct action_data *args)
 {  
   GtkWidget *sw, *label, *vbox, *view;
   GtkListStore *store;
@@ -413,19 +487,14 @@ static void create_improvements_list(struct player *pplayer,
 				  GTK_RESPONSE_ACCEPT);
 
   label = gtk_frame_new(_("Select Improvement to Sabotage"));
-  gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area(GTK_DIALOG(spy_sabotage_shell))), label);
+  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(spy_sabotage_shell)->vbox), label);
 
-  vbox = gtk_grid_new();
-  gtk_orientable_set_orientation(GTK_ORIENTABLE(vbox),
-                                 GTK_ORIENTATION_VERTICAL);
-  gtk_grid_set_row_spacing(GTK_GRID(vbox), 6);
+  vbox = gtk_vbox_new(FALSE, 6);
   gtk_container_add(GTK_CONTAINER(label), vbox);
       
   store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
 
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
-  gtk_widget_set_hexpand(view, TRUE);
-  gtk_widget_set_vexpand(view, TRUE);
   g_object_unref(store);
   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
 
@@ -450,7 +519,7 @@ static void create_improvements_list(struct player *pplayer,
 
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
     GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
-  gtk_scrolled_window_set_min_content_height(GTK_SCROLLED_WINDOW(sw), 200);
+  gtk_widget_set_size_request(sw, -1, 200);
   
   gtk_container_add(GTK_CONTAINER(vbox), sw);
 
@@ -473,7 +542,7 @@ static void create_improvements_list(struct player *pplayer,
     struct astring str = ASTRING_INIT;
     /* TRANS: %s is a unit name, e.g., Spy */
     astr_set(&str, _("At %s's Discretion"),
-             unit_name_translation(game_unit_by_number(diplomat_id)));
+             unit_name_translation(game_unit_by_number(args->actor_unit_id)));
     gtk_list_store_set(store, &it, 0, astr_str(&str), 1, B_LAST, -1);
     astr_free(&str);
   }
@@ -481,14 +550,14 @@ static void create_improvements_list(struct player *pplayer,
   gtk_dialog_set_response_sensitive(GTK_DIALOG(spy_sabotage_shell),
     GTK_RESPONSE_ACCEPT, FALSE);
   
-  gtk_widget_show_all(gtk_dialog_get_content_area(GTK_DIALOG(spy_sabotage_shell)));
+  gtk_widget_show_all(GTK_DIALOG(spy_sabotage_shell)->vbox);
 
   g_signal_connect(gtk_tree_view_get_selection(GTK_TREE_VIEW(view)), "changed",
-		   G_CALLBACK(spy_improvements_callback), NULL);
+                   G_CALLBACK(spy_improvements_callback), args);
   g_signal_connect(spy_sabotage_shell, "response",
-		   G_CALLBACK(spy_improvements_response), NULL);
+                   G_CALLBACK(spy_improvements_response), args);
 
-  sabotage_improvement = -2;
+  args->value = -2;
 	  
   gtk_tree_view_focus(GTK_TREE_VIEW(view));
 }
@@ -498,20 +567,27 @@ static void create_improvements_list(struct player *pplayer,
 *****************************************************************/
 static void spy_steal_popup(GtkWidget *w, gpointer data)
 {
-  struct city *pvcity = game_city_by_number(diplomat_target_id);
+  struct action_data *args = (struct action_data *)data;
+
+  struct city *pvcity = game_city_by_number(args->target_city_id);
   struct player *pvictim = NULL;
 
-  if(pvcity)
+  if (pvcity) {
     pvictim = city_owner(pvcity);
+  }
 
 /* it is concievable that pvcity will not be found, because something
 has happened to the city during latency.  Therefore we must initialize
 pvictim to NULL and account for !pvictim in create_advances_list. -- Syela */
-  
-  if(!spy_tech_shell){
-    create_advances_list(client.conn.playing, pvictim);
+
+  /* FIXME: Don't discard the second tech choice dialog. */
+  if (!spy_tech_shell) {
+    create_advances_list(client.conn.playing, pvictim, args);
     gtk_window_present(GTK_WINDOW(spy_tech_shell));
+  } else {
+    free(args);
   }
+
   gtk_widget_destroy(diplomat_dialog);
 }
 
@@ -521,22 +597,28 @@ pvictim to NULL and account for !pvictim in create_advances_list. -- Syela */
 *****************************************************************/
 static void spy_request_sabotage_list(GtkWidget *w, gpointer data)
 {
-  if (NULL != game_unit_by_number(diplomat_id)
-      && NULL != game_city_by_number(diplomat_target_id)) {
-    request_diplomat_answer(DIPLOMAT_SABOTAGE, diplomat_id,
-                            diplomat_target_id, 0);
+  struct action_data *args = (struct action_data *)data;
+
+  if (NULL != game_unit_by_number(args->actor_unit_id)
+      && NULL != game_city_by_number(args->target_city_id)) {
+    request_diplomat_answer(DIPLOMAT_SABOTAGE, args->actor_unit_id,
+                            args->target_city_id, 0);
   }
+
   gtk_widget_destroy(diplomat_dialog);
+  free(args);
 }
 
-/****************************************************************
+/*************************************************************************
  Pops-up the Spy sabotage dialog, upon return of list of
  available improvements requested by the above function.
-*****************************************************************/
-void popup_sabotage_dialog(struct city *pcity)
+**************************************************************************/
+void popup_sabotage_dialog(struct unit *actor, struct city *pcity)
 {
-  if(!spy_sabotage_shell){
-    create_improvements_list(client.conn.playing, pcity);
+  /* FIXME: Don't discard the second target choice dialog. */
+  if (!spy_sabotage_shell) {
+    create_improvements_list(client.conn.playing, pcity,
+                             act_data(actor->id, pcity->id, 0, 0));
     gtk_window_present(GTK_WINDOW(spy_sabotage_shell));
   }
 }
@@ -546,30 +628,38 @@ void popup_sabotage_dialog(struct city *pcity)
 *****************************************************************/
 static void diplomat_incite_callback(GtkWidget *w, gpointer data)
 {
-  if (NULL != game_unit_by_number(diplomat_id)
-      && NULL != game_city_by_number(diplomat_target_id)) {
-    request_diplomat_answer(DIPLOMAT_INCITE, diplomat_id,
-                            diplomat_target_id, 0);
+  struct action_data *args = (struct action_data *)data;
+
+  if (NULL != game_unit_by_number(args->actor_unit_id)
+      && NULL != game_city_by_number(args->target_city_id)) {
+    request_diplomat_answer(DIPLOMAT_INCITE, args->actor_unit_id,
+                           args->target_city_id, 0);
   }
+
   gtk_widget_destroy(diplomat_dialog);
+  free(args);
 }
 
-/****************************************************************
+/************************************************************************
   User has responded to incite dialog
-*****************************************************************/
-static void incite_response(GtkWidget *w, gint response)
+************************************************************************/
+static void incite_response(GtkWidget *w, gint response, gpointer data)
 {
+  struct action_data *args = (struct action_data *)data;
+
   if (response == GTK_RESPONSE_YES) {
-    request_diplomat_action(DIPLOMAT_INCITE, diplomat_id,
-			    diplomat_target_id, 0);
+    request_diplomat_action(DIPLOMAT_INCITE, args->actor_unit_id,
+                            args->target_city_id, 0);
   }
+
   gtk_widget_destroy(w);
+  free(args);
 }
 
-/****************************************************************
+/*************************************************************************
 Popup the yes/no dialog for inciting, since we know the cost now
-*****************************************************************/
-void popup_incite_dialog(struct city *pcity, int cost)
+**************************************************************************/
+void popup_incite_dialog(struct unit *actor, struct city *pcity, int cost)
 {
   GtkWidget *shell;
   char buf[1024];
@@ -607,7 +697,8 @@ void popup_incite_dialog(struct city *pcity, int cost)
   }
   gtk_window_present(GTK_WINDOW(shell));
   
-  g_signal_connect(shell, "response", G_CALLBACK(incite_response), NULL);
+  g_signal_connect(shell, "response", G_CALLBACK(incite_response),
+                   act_data(actor->id, pcity->id, 0, cost));
 }
 
 
@@ -617,16 +708,20 @@ void popup_incite_dialog(struct city *pcity, int cost)
 *****************************************************************/
 static void diplomat_keep_moving_callback(GtkWidget *w, gpointer data)
 {
+  struct action_data *args = (struct action_data *)data;
+
   struct unit *punit;
   struct city *pcity;
 
-  if ((punit = game_unit_by_number(diplomat_id))
-      && (pcity = game_city_by_number(diplomat_target_id))
+  if ((punit = game_unit_by_number(args->actor_unit_id))
+      && (pcity = game_city_by_number(args->target_city_id))
       && !same_pos(unit_tile(punit), city_tile(pcity))) {
-    request_diplomat_action(DIPLOMAT_MOVE, diplomat_id,
-                            diplomat_target_id, 0);
+    request_diplomat_action(DIPLOMAT_MOVE, args->actor_unit_id,
+                            args->target_city_id, 0);
   }
+
   gtk_widget_destroy(diplomat_dialog);
+  free(args);
 }
 
 /****************************************************************
@@ -644,6 +739,18 @@ static void diplomat_destroy_callback(GtkWidget *w, gpointer data)
 static void diplomat_cancel_callback(GtkWidget *w, gpointer data)
 {
   gtk_widget_destroy(diplomat_dialog);
+  free(data);
+}
+
+/****************************************************************
+  Diplomat dialog has been closed
+*****************************************************************/
+static void diplomat_close_callback(GtkWidget *w,
+                                    gint response_id,
+                                    gpointer data)
+{
+  gtk_widget_destroy(diplomat_dialog);
+  free(data);
 }
 
 /****************************************************************
@@ -661,7 +768,8 @@ void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
   if ((pcity = tile_city(dest_tile))) {
     /* Spy/Diplomat acting against a city */
 
-    diplomat_target_id = pcity->id;
+    struct action_data *data = act_data(punit->id, pcity->id, 0, 0);
+
     astr_set(&title,
              /* TRANS: %s is a unit name, e.g., Spy */
              _("Choose Your %s's Strategy"), unit_name_translation(punit));
@@ -673,13 +781,13 @@ void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
     if (!unit_has_type_flag(punit, F_SPY)){
       shl = popup_choice_dialog(GTK_WINDOW(toplevel),
         astr_str(&title), astr_str(&text),
-	_("Establish _Embassy"), diplomat_embassy_callback, NULL,
-	_("_Investigate City"), diplomat_investigate_callback, NULL,
-	_("_Sabotage City"), diplomat_sabotage_callback, NULL,
-	_("Steal _Technology"), diplomat_steal_callback, NULL,
-	_("Incite a _Revolt"), diplomat_incite_callback, NULL,
-	_("_Keep moving"), diplomat_keep_moving_callback, NULL,
-	GTK_STOCK_CANCEL, diplomat_cancel_callback, NULL,
+        _("Establish _Embassy"), diplomat_embassy_callback, data,
+        _("_Investigate City"), diplomat_investigate_callback, data,
+        _("_Sabotage City"), diplomat_sabotage_callback, data,
+        _("Steal _Technology"), diplomat_steal_callback, data,
+        _("Incite a _Revolt"), diplomat_incite_callback, data,
+        _("_Keep moving"), diplomat_keep_moving_callback, data,
+        GTK_STOCK_CANCEL, diplomat_cancel_callback, data,
 	NULL);
 
       if (!diplomat_can_do_action(punit, DIPLOMAT_EMBASSY, dest_tile))
@@ -692,19 +800,20 @@ void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
 	choice_dialog_button_set_sensitive(shl, 3, FALSE);
       if (!diplomat_can_do_action(punit, DIPLOMAT_INCITE, dest_tile))
 	choice_dialog_button_set_sensitive(shl, 4, FALSE);
-      if (!diplomat_can_do_action(punit, DIPLOMAT_MOVE, dest_tile))
+      if (!unit_can_move_to_tile(punit, dest_tile, FALSE)) {
 	choice_dialog_button_set_sensitive(shl, 5, FALSE);
+      }
     } else {
        shl = popup_choice_dialog(GTK_WINDOW(toplevel),
         astr_str(&title), astr_str(&text),
-	_("Establish _Embassy"), diplomat_embassy_callback, NULL,
-	_("_Investigate City"), diplomat_investigate_callback, NULL,
-	_("_Poison City"), spy_poison_callback, NULL,
-	_("Industrial _Sabotage"), spy_request_sabotage_list, NULL,
-	_("Steal _Technology"), spy_steal_popup, NULL,
-	_("Incite a _Revolt"), diplomat_incite_callback, NULL,
-	_("_Keep moving"), diplomat_keep_moving_callback, NULL,
-	GTK_STOCK_CANCEL, diplomat_cancel_callback, NULL,
+        _("Establish _Embassy"), diplomat_embassy_callback, data,
+        _("_Investigate City"), diplomat_investigate_callback, data,
+        _("_Poison City"), spy_poison_callback, data,
+        _("Industrial _Sabotage"), spy_request_sabotage_list, data,
+        _("Steal _Technology"), spy_steal_popup, data,
+        _("Incite a _Revolt"), diplomat_incite_callback, data,
+        _("_Keep moving"), diplomat_keep_moving_callback, data,
+        GTK_STOCK_CANCEL, diplomat_cancel_callback, data,
 	NULL);
 
       if (!diplomat_can_do_action(punit, DIPLOMAT_EMBASSY, dest_tile))
@@ -719,8 +828,9 @@ void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
 	choice_dialog_button_set_sensitive(shl, 4, FALSE);
       if (!diplomat_can_do_action(punit, DIPLOMAT_INCITE, dest_tile))
 	choice_dialog_button_set_sensitive(shl, 5, FALSE);
-      if (!diplomat_can_do_action(punit, DIPLOMAT_MOVE, dest_tile))
+      if (!unit_can_move_to_tile(punit, dest_tile, FALSE)) {
 	choice_dialog_button_set_sensitive(shl, 6, FALSE);
+      }
      }
 
     diplomat_dialog = shl;
@@ -729,12 +839,13 @@ void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
     g_signal_connect(shl, "destroy",
 		     G_CALLBACK(diplomat_destroy_callback), NULL);
     g_signal_connect(shl, "delete_event",
-		     G_CALLBACK(diplomat_cancel_callback), NULL);
+		     G_CALLBACK(diplomat_close_callback), data);
   } else { 
     if ((ptunit = unit_list_get(dest_tile->units, 0))){
       /* Spy/Diplomat acting against a unit */ 
+
+      struct action_data *data = act_data(punit->id, 0, ptunit->id, 0);
        
-      diplomat_target_id = ptunit->id;
       astr_set(&text,
                /* TRANS: %s is a unit name, e.g., Diplomat, Spy */
                _("Your %s is waiting for your command."),
@@ -742,9 +853,9 @@ void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
  
       shl = popup_choice_dialog(GTK_WINDOW(toplevel),
 	_("Subvert Enemy Unit"), astr_str(&text),
-	_("_Bribe Enemy Unit"), diplomat_bribe_callback, NULL,
-	_("_Sabotage Enemy Unit"), spy_sabotage_unit_callback, NULL,
-	GTK_STOCK_CANCEL, diplomat_cancel_callback, NULL,
+	_("_Bribe Enemy Unit"), diplomat_bribe_callback, data,
+	_("_Sabotage Enemy Unit"), spy_sabotage_unit_callback, data,
+	GTK_STOCK_CANCEL, diplomat_cancel_callback, data,
 	NULL);
 
       if (!diplomat_can_do_action(punit, DIPLOMAT_BRIBE, dest_tile)) {
@@ -760,7 +871,7 @@ void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
       g_signal_connect(shl, "destroy",
 		       G_CALLBACK(diplomat_destroy_callback), NULL);
       g_signal_connect(shl, "delete_event",
-		       G_CALLBACK(diplomat_cancel_callback), NULL);
+		       G_CALLBACK(diplomat_close_callback), data);
     }
   }
   astr_free(&title);
