@@ -40,7 +40,7 @@
 #include "options.h"
 #include "tilespec.h"
 
-/* client/gui-gtk-2.0 */
+/* client/gui-gtk-3.0 */
 #include "chatline.h"
 #include "cityrep.h"
 #include "dialogs.h"
@@ -780,8 +780,8 @@ static void show_fog_of_war_callback(GtkToggleAction *action, gpointer data)
 static void show_better_fog_of_war_callback(GtkToggleAction *action,
                                             gpointer data)
 {
-  if (gui_gtk2_better_fog ^ gtk_toggle_action_get_active(action)) {
-    gui_gtk2_better_fog ^= 1;
+  if (gui_gtk3_better_fog ^ gtk_toggle_action_get_active(action)) {
+    gui_gtk3_better_fog ^= 1;
     update_map_canvas_visible();
   }
 }
@@ -1322,7 +1322,7 @@ static GtkActionGroup *get_safe_group(void)
        NULL, NULL, G_CALLBACK(save_game_callback)},
       {"GAME_SAVE_AS", GTK_STOCK_SAVE_AS, _("Save Game _As..."),
        NULL, NULL, G_CALLBACK(save_game_as_callback)},
-      {"MAPIMG_SAVE", NULL, _("Save Map _Image"),
+     {"MAPIMG_SAVE", NULL, _("Save Map _Image"),
        NULL, NULL, G_CALLBACK(save_mapimg_callback)},
       {"MAPIMG_SAVE_AS", NULL, _("Save _Map Image As..."),
        NULL, NULL, G_CALLBACK(save_mapimg_as_callback)},
@@ -1746,7 +1746,7 @@ static const gchar *get_ui_filename(void)
 static void add_widget_callback(GtkUIManager *ui_manager, GtkWidget *widget,
                                 gpointer data)
 {
-  gtk_box_pack_start(GTK_BOX(data), widget, TRUE, TRUE, 0);
+  gtk_container_add(GTK_CONTAINER(data), widget);
   gtk_widget_show(widget);
 }
 
@@ -1755,7 +1755,7 @@ static void add_widget_callback(GtkUIManager *ui_manager, GtkWidget *widget,
 *****************************************************************/
 GtkWidget *setup_menus(GtkWidget *window)
 {
-  GtkWidget *menubar = gtk_hbox_new(FALSE, 0);
+  GtkWidget *menubar = gtk_grid_new();
   GError *error = NULL;
 
   /* Creates the UI manager. */
@@ -1872,8 +1872,7 @@ static void menus_rename(GtkActionGroup *group,
     return;
   }
 
-  /* gtk_action_set_label() was added in Gtk 2.16. */
-  g_object_set(G_OBJECT(action), "label", new_label, NULL);
+  gtk_action_set_label(action, new_label);
 }
 
 /****************************************************************
@@ -1942,7 +1941,7 @@ static const char *get_tile_change_menu_text(struct tile *ptile,
   const char *text;
 
   tile_apply_activity(newtile, activity);
-  text = tile_get_info_text(newtile, 0);
+  text = tile_get_info_text(newtile, FALSE, 0);
   tile_virtual_destroy(newtile);
   return text;
 }
@@ -2006,7 +2005,7 @@ void real_menus_update(void)
   {
     char road_buf[500];
 
-    /* TRANS: Connect with some road type (Road/Railroad) */  
+    /* TRANS: Connect with some road type (Road/Railroad) */
     snprintf(road_buf, sizeof(road_buf), _("Connect With %s"),
              road_name_translation(road_by_number(ROAD_ROAD)));
     menus_rename(unit_group, "CONNECT_ROAD", road_buf);
@@ -2351,7 +2350,7 @@ void real_menus_init(void)
         g_object_set_data(G_OBJECT(item), "government", g);
 
         if ((gsprite = get_government_sprite(tileset, g))) {
-          image = gtk_image_new_from_pixbuf(sprite_get_pixbuf(gsprite));
+          image = gtk_pixcomm_new_from_sprite(gsprite);
           gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
           gtk_widget_show(image);
         }
@@ -2427,7 +2426,7 @@ void real_menus_init(void)
   if (tileset_use_hard_coded_fog(tileset)) {
     menus_set_visible(safe_group, "SHOW_BETTER_FOG_OF_WAR", TRUE, TRUE);
     menus_set_active(safe_group, "SHOW_BETTER_FOG_OF_WAR",
-                     gui_gtk2_better_fog);
+                     gui_gtk3_better_fog);
   } else {
     menus_set_visible(safe_group, "SHOW_BETTER_FOG_OF_WAR", FALSE, FALSE);
   }
