@@ -50,7 +50,7 @@
 #include "reqtree.h"
 #include "text.h"
 
-/* client/gui-gtk-2.0 */
+/* client/gui-gtk-3.0 */
 #include "canvas.h"
 #include "cityrep.h"
 #include "dialogs.h"
@@ -86,9 +86,10 @@ static bool science_report_combo_get_active(GtkComboBox *combo,
 static void science_report_combo_set_active(GtkComboBox *combo,
                                             Tech_type_id tech);
 static gboolean science_diagram_button_release_callback(GtkWidget *widget,
-                                                        GdkEventButton *event,
-                                                        gpointer data);
-static void science_diagram_update(GtkWidget *widget, gpointer data);
+    GdkEventButton *event, gpointer data);
+static gboolean science_diagram_update(GtkWidget *widget,
+                                       cairo_t *cr,
+                                       gpointer data);
 static GtkWidget *science_diagram_new(void);
 static void science_diagram_data(GtkWidget *widget, bool reachable);
 static void science_diagram_center(GtkWidget *diagram, Tech_type_id tech);
@@ -225,18 +226,31 @@ static gboolean science_diagram_button_release_callback(GtkWidget *widget,
 /****************************************************************************
   Draw the invalidated portion of the reqtree.
 ****************************************************************************/
-static void science_diagram_update(GtkWidget *widget, gpointer data)
+static gboolean science_diagram_update(GtkWidget *widget, cairo_t *cr, gpointer data)
 {
   /* FIXME: this currently redraws everything! */
-  struct canvas canvas = {
-    .type = CANVAS_PIXMAP,
-    .v.pixmap = GTK_LAYOUT(widget)->bin_window
-  };
+  struct canvas canvas = FC_STATIC_CANVAS_INIT;
   struct reqtree *reqtree = g_object_get_data(G_OBJECT(widget), "reqtree");
   int width, height;
+  GtkAdjustment *hadjustment;
+  GtkAdjustment *vadjustment;
+  gint hadjustment_value;
+  gint vadjustment_value;
+
+  hadjustment = gtk_scrollable_get_hadjustment(GTK_SCROLLABLE(widget));
+  vadjustment = gtk_scrollable_get_vadjustment(GTK_SCROLLABLE(widget));
+
+  hadjustment_value = (gint)gtk_adjustment_get_value(hadjustment);
+  vadjustment_value = (gint)gtk_adjustment_get_value(vadjustment);
+
+  cairo_translate(cr, -hadjustment_value, -vadjustment_value);
+
+  canvas.drawable = cr;
 
   get_reqtree_dimensions(reqtree, &width, &height);
   draw_reqtree(reqtree, &canvas, 0, 0, 0, 0, width, height);
+
+  return TRUE;
 }
 
 /****************************************************************************
@@ -251,7 +265,7 @@ static GtkWidget *science_diagram_new(void)
   gtk_widget_add_events(diagram,
                         GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
                         | GDK_BUTTON2_MOTION_MASK | GDK_BUTTON3_MOTION_MASK);
-  g_signal_connect(diagram, "expose-event",
+  g_signal_connect(diagram, "draw",
                    G_CALLBACK(science_diagram_update), NULL);
   g_signal_connect(diagram, "button-release-event",
                    G_CALLBACK(science_diagram_button_release_callback),
@@ -302,12 +316,16 @@ static void science_diagram_center(GtkWidget *diagram, Tech_type_id tech)
     gdouble value;
 
     adjust = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(sw));
-    value = (adjust->lower + adjust->upper - adjust->page_size) / width * x;
+    value = (gtk_adjustment_get_lower(adjust)
+      + gtk_adjustment_get_upper(adjust)
+      - gtk_adjustment_get_page_size(adjust)) / width * x;
     gtk_adjustment_set_value(adjust, value);
     gtk_adjustment_value_changed(adjust);
 
     adjust = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(sw));
-    value = (adjust->lower + adjust->upper - adjust->page_size) / height * y;
+    value = (gtk_adjustment_get_lower(adjust)
+      + gtk_adjustment_get_upper(adjust)
+      - gtk_adjustment_get_page_size(adjust)) / height * y;
     gtk_adjustment_set_value(adjust, value);
     gtk_adjustment_value_changed(adjust);
   }
@@ -366,6 +384,9 @@ static void science_report_update(struct science_report *preport)
   fc_assert_ret(NULL != preport);
   fc_assert_ret(NULL != presearch);
 
+  /* Disable callbacks. */
+  science_report_no_combo_callback = TRUE;
+
   gtk_widget_queue_draw(GTK_WIDGET(preport->drawing_area));
 
   gtk_label_set_text(preport->main_label, science_dialog_text());
@@ -445,6 +466,9 @@ static void science_report_update(struct science_report *preport)
 
   /* Free. */
   g_list_free(sorting_list);
+
+  /* Re-enable callbacks. */
+  science_report_no_combo_callback = FALSE;
 }
 
 /****************************************************************************
@@ -509,7 +533,8 @@ static void science_report_goal_callback(GtkComboBox *combo, gpointer data)
 static void science_report_init(struct science_report *preport)
 {
   GtkWidget *frame, *table, *help_button, *reachable_button, *sw, *w;
-  GtkBox *vbox;
+  GtkSizeGroup *group;
+  GtkContainer *vbox;
   GtkListStore *store;
   GtkCellRenderer *renderer;
 
@@ -522,25 +547,27 @@ static void science_report_init(struct science_report *preport)
   gui_dialog_add_button(preport->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
   gui_dialog_set_default_response(preport->shell, GTK_RESPONSE_CLOSE);
 
-  vbox = GTK_BOX(preport->shell->vbox);
+  vbox = GTK_CONTAINER(preport->shell->vbox);
+  group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
 
   w = gtk_label_new(NULL);
-  gtk_box_pack_start(vbox, w, FALSE, FALSE, 0);
+  gtk_container_add(vbox, w);
   preport->main_label = GTK_LABEL(w);
 
   /* Current research target line. */
   frame = gtk_frame_new(_("Researching"));
-  gtk_box_pack_start(vbox, frame, FALSE, FALSE, 0);
+  gtk_container_add(vbox, frame);
 
-  table = gtk_table_new(1, 6, TRUE);
-  gtk_table_set_col_spacings(GTK_TABLE(table), 4);
+  table = gtk_grid_new();
+  gtk_grid_set_column_spacing(GTK_GRID(table), 4);
   gtk_container_add(GTK_CONTAINER(frame), table);
 
   help_button = gtk_check_button_new_with_label(_("Help"));
-  gtk_table_attach(GTK_TABLE(table), help_button, 5, 6, 0, 1, 0, 0, 0, 0);
+  gtk_grid_attach(GTK_GRID(table), help_button, 5, 0, 1, 1);
 
   store = science_report_store_new();
   w = gtk_combo_box_new_with_model(GTK_TREE_MODEL(store));
+  gtk_size_group_add_widget(group, w);
   g_object_unref(G_OBJECT(store));
   renderer = gtk_cell_renderer_text_new();
   gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(w), renderer, TRUE);
@@ -549,24 +576,27 @@ static void science_report_init(struct science_report *preport)
   gtk_widget_set_sensitive(w, can_client_issue_orders());
   g_signal_connect(w, "changed", G_CALLBACK(science_report_current_callback),
                    help_button);
-  gtk_table_attach_defaults(GTK_TABLE(table), w, 0, 2, 0, 1);
+  gtk_grid_attach(GTK_GRID(table), w, 0, 0, 1, 1);
   preport->reachable_techs = GTK_COMBO_BOX(w);
 
   w = gtk_progress_bar_new();
-  gtk_table_attach_defaults(GTK_TABLE(table), w, 2, 5, 0, 1);
+  gtk_widget_set_hexpand(w, TRUE);
+  gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(w), TRUE);
+  gtk_grid_attach(GTK_GRID(table), w, 2, 0, 1, 1);
   gtk_widget_set_size_request(w, -1, 25);
   preport->progress_bar = GTK_PROGRESS_BAR(w);
 
   /* Research goal line. */
   frame = gtk_frame_new( _("Goal"));
-  gtk_box_pack_start(vbox, frame, FALSE, FALSE, 0);
+  gtk_container_add(vbox, frame);
 
-  table = gtk_table_new(1, 6, TRUE);
-  gtk_table_set_col_spacings(GTK_TABLE(table), 4);
+  table = gtk_grid_new();
+  gtk_grid_set_column_spacing(GTK_GRID(table), 4);
   gtk_container_add(GTK_CONTAINER(frame),table);
 
   store = science_report_store_new();
   w = gtk_combo_box_new_with_model(GTK_TREE_MODEL(store));
+  gtk_size_group_add_widget(group, w);
   g_object_unref(G_OBJECT(store));
   renderer = gtk_cell_renderer_text_new();
   gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(w), renderer, TRUE);
@@ -579,19 +609,19 @@ static void science_report_init(struct science_report *preport)
   gtk_widget_set_sensitive(w, can_client_issue_orders());
   g_signal_connect(w, "changed", G_CALLBACK(science_report_goal_callback),
                    help_button);
-  gtk_table_attach_defaults(GTK_TABLE(table), w, 0, 2, 0, 1);
+  gtk_grid_attach(GTK_GRID(table), w, 0, 0, 1, 1);
   preport->reachable_goals = GTK_COMBO_BOX(w);
 
   w = gtk_label_new(NULL);
-  gtk_table_attach_defaults(GTK_TABLE(table), w, 2, 5, 0, 1);
+  gtk_widget_set_hexpand(w, TRUE);
+  gtk_grid_attach(GTK_GRID(table), w, 2, 0, 1, 1);
   gtk_widget_set_size_request(w, -1, 25);
   preport->goal_label = GTK_LABEL(w);
 
   /* Toggle unreachable button. */
   /* TRANS: As in 'Show all (even currently not reachable) techs'. */
   reachable_button = gtk_toggle_button_new_with_label(_("Show all"));
-  gtk_table_attach(GTK_TABLE(table), reachable_button, 5, 6, 0, 1, 0, 0, 0,
-                   0);
+  gtk_grid_attach(GTK_GRID(table), reachable_button, 5, 0, 1, 1);
   g_signal_connect(reachable_button, "toggled",
                    G_CALLBACK(science_report_unreachable_callback), preport);
   gtk_widget_set_sensitive(reachable_button, can_client_issue_orders()
@@ -602,14 +632,18 @@ static void science_report_init(struct science_report *preport)
   sw = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_box_pack_start(vbox, sw, TRUE, TRUE, 0);
+  gtk_container_add(vbox, sw);
 
   w = science_diagram_new();
+  gtk_widget_set_hexpand(w, TRUE);
+  gtk_widget_set_vexpand(w, TRUE);
   gtk_container_add(GTK_CONTAINER(sw), w);
   preport->drawing_area = GTK_LAYOUT(w);
 
   science_report_update(preport);
   gui_dialog_show_all(preport->shell);
+  gtk_widget_queue_draw(GTK_WIDGET(preport->drawing_area));
+  g_object_unref(group);
 
   /* This must be _after_ the dialog is drawn to really center it ... */
   science_report_redraw(preport);
@@ -694,7 +728,7 @@ struct economy_report {
   GtkLabel *label;
 };
 
-static struct economy_report economy_report = { NULL, };
+static struct economy_report economy_report = { NULL, NULL, NULL };
 
 enum economy_report_response {
   ERD_RES_SELL_REDUNDANT = 1,
@@ -775,6 +809,7 @@ static void economy_report_update(struct economy_report *preport)
   GtkTreeModel *model;
   GtkListStore *store;
   GtkTreeIter iter;
+  GdkPixbuf *pix;
   struct improvement_entry building_entries[B_LAST];
   struct unit_entry unit_entries[U_LAST];
   int entries_used, building_total, unit_total, tax, i;
@@ -804,9 +839,10 @@ static void economy_report_update(struct economy_report *preport)
     struct sprite *sprite = get_building_sprite(tileset, pimprove);
     cid cid = cid_encode_building(pimprove);
 
+    pix = sprite_get_pixbuf(sprite);
     gtk_list_store_append(store, &iter);
     gtk_list_store_set(store, &iter,
-                       ERD_COL_SPRITE, sprite_get_pixbuf(sprite),
+                       ERD_COL_SPRITE, pix,
                        ERD_COL_NAME, improvement_name_translation(pimprove),
                        ERD_COL_REDUNDANT, pentry->redundant,
                        ERD_COL_COUNT, pentry->count,
@@ -815,6 +851,7 @@ static void economy_report_update(struct economy_report *preport)
                        ERD_COL_IS_IMPROVEMENT, TRUE,
                        ERD_COL_CID, cid,
                        -1);
+    g_object_unref(G_OBJECT(pix));
     if (selected == cid) {
       /* Restore the selection. */
       gtk_tree_selection_select_iter(selection, &iter);
@@ -830,9 +867,10 @@ static void economy_report_update(struct economy_report *preport)
                                                 direction8_invalid(), TRUE);
     cid cid = cid_encode_unit(putype);
 
+    pix = sprite_get_pixbuf(sprite);
     gtk_list_store_append(store, &iter);
     gtk_list_store_set(store, &iter,
-                       ERD_COL_SPRITE, sprite_get_pixbuf(sprite),
+                       ERD_COL_SPRITE, pix,
                        ERD_COL_NAME, utype_name_translation(putype),
                        ERD_COL_REDUNDANT, 0,
                        ERD_COL_COUNT, pentry->count,
@@ -841,6 +879,7 @@ static void economy_report_update(struct economy_report *preport)
                        ERD_COL_IS_IMPROVEMENT, FALSE,
                        ERD_COL_CID, cid,
                        -1);
+    g_object_unref(G_OBJECT(pix));
     if (selected == cid) {
       /* Restore the selection. */
       gtk_tree_selection_select_iter(selection, &iter);
@@ -1022,10 +1061,10 @@ static void economy_report_selection_callback(GtkTreeSelection *selection,
 ****************************************************************************/
 static void economy_report_init(struct economy_report *preport)
 {
-  GtkWidget *view, *sw, *align, *label, *button;
+  GtkWidget *view, *sw, *label, *button;
   GtkListStore *store;
   GtkTreeSelection *selection;
-  GtkBox *vbox;
+  GtkContainer *vbox;
   const char *title;
   enum economy_report_columns i;
 
@@ -1033,20 +1072,19 @@ static void economy_report_init(struct economy_report *preport)
 
   gui_dialog_new(&preport->shell, GTK_NOTEBOOK(top_notebook), preport, TRUE);
   gui_dialog_set_title(preport->shell, _("Economy"));
-  vbox = GTK_BOX(preport->shell->vbox);
-
-  align = gtk_alignment_new(0.5, 0.0, 0.0, 1.0);
-  gtk_box_pack_start(vbox, align, TRUE, TRUE, 0);
+  vbox = GTK_CONTAINER(preport->shell->vbox);
 
   sw = gtk_scrolled_window_new(NULL, NULL);
+  gtk_widget_set_halign(sw, GTK_ALIGN_CENTER);
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
                                       GTK_SHADOW_ETCHED_IN);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  gtk_container_add(GTK_CONTAINER(align), sw);
+  gtk_container_add(GTK_CONTAINER(vbox), sw);
 
   store = economy_report_store_new();
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+  gtk_widget_set_vexpand(view, TRUE);
   g_object_unref(store);
   gtk_widget_set_name(view, "small_font");
   gtk_tree_view_columns_autosize(GTK_TREE_VIEW(view));
@@ -1097,24 +1135,24 @@ static void economy_report_init(struct economy_report *preport)
   }
 
   label = gtk_label_new(NULL);
-  gtk_box_pack_start(vbox, label, FALSE, FALSE, 0);
+  gtk_container_add(vbox, label);
   gtk_misc_set_padding(GTK_MISC(label), 5, 5);
   preport->label = GTK_LABEL(label);
 
-  button = gui_dialog_add_button(preport->shell, _("Sell _Redundant"),
-                                 ERD_RES_SELL_REDUNDANT);
+  gui_dialog_add_button(preport->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+
+  button = gui_dialog_add_button(preport->shell, _("_Disband"),
+                                 ERD_RES_DISBAND_UNITS);
   gtk_widget_set_sensitive(button, FALSE);
 
   button = gui_dialog_add_button(preport->shell, _("Sell _All"),
                                  ERD_RES_SELL_ALL);
   gtk_widget_set_sensitive(button, FALSE);
 
-  button = gui_dialog_add_button(preport->shell, _("_Disband"),
-                                 ERD_RES_DISBAND_UNITS);
+  button = gui_dialog_add_button(preport->shell, _("Sell _Redundant"),
+                                 ERD_RES_SELL_REDUNDANT);
   gtk_widget_set_sensitive(button, FALSE);
 
-  gui_dialog_add_button(preport->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
-
   gui_dialog_set_default_response(preport->shell, GTK_RESPONSE_CLOSE);
   gui_dialog_response_set_callback(preport->shell,
                                    economy_report_command_callback);
@@ -1184,7 +1222,7 @@ struct units_report {
   GtkTreeView *tree_view;
 };
 
-static struct units_report units_report = { NULL, };
+static struct units_report units_report = { NULL, NULL };
 
 enum units_report_response {
   URD_RES_NEAREST = 1,
@@ -1547,10 +1585,10 @@ static void units_report_command_callback(struct gui_dialog *pdialog,
 ****************************************************************************/
 static void units_report_init(struct units_report *preport)
 {
-  GtkWidget *view, *sw, *align, *button;
+  GtkWidget *view, *sw, *button;
   GtkListStore *store;
   GtkTreeSelection *selection;
-  GtkBox *vbox;
+  GtkContainer *vbox;
   GtkTreeViewColumn *col = NULL;
   enum units_report_columns i;
 
@@ -1558,20 +1596,19 @@ static void units_report_init(struct units_report *preport)
 
   gui_dialog_new(&preport->shell, GTK_NOTEBOOK(top_notebook), preport, TRUE);
   gui_dialog_set_title(preport->shell, _("Units"));
-  vbox = GTK_BOX(preport->shell->vbox);
-
-  align = gtk_alignment_new(0.5, 0.0, 0.0, 1.0);
-  gtk_box_pack_start(vbox, align, TRUE, TRUE, 0);
+  vbox = GTK_CONTAINER(preport->shell->vbox);
 
   sw = gtk_scrolled_window_new(NULL,NULL);
+  gtk_widget_set_halign(sw, GTK_ALIGN_CENTER);
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
                                       GTK_SHADOW_ETCHED_IN);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  gtk_container_add(GTK_CONTAINER(align), sw);
+  gtk_container_add(GTK_CONTAINER(vbox), sw);
 
   store = units_report_store_new();
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+  gtk_widget_set_vexpand(view, TRUE);
   g_object_unref(store);
   gtk_widget_set_name(view, "small_font");
   gtk_tree_view_columns_autosize(GTK_TREE_VIEW(view));
@@ -1623,15 +1660,15 @@ static void units_report_init(struct units_report *preport)
     }
   }
 
-  button = gui_dialog_add_stockbutton(preport->shell, GTK_STOCK_FIND,
-                                      _("Find _Nearest"), URD_RES_NEAREST);
-  gtk_widget_set_sensitive(button, FALSE);
+  gui_dialog_add_button(preport->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 
   button = gui_dialog_add_button(preport->shell, _("_Upgrade"),
                                  URD_RES_UPGRADE);
   gtk_widget_set_sensitive(button, FALSE);
 
-  gui_dialog_add_button(preport->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+  button = gui_dialog_add_stockbutton(preport->shell, GTK_STOCK_FIND,
+                                      _("Find _Nearest"), URD_RES_NEAREST);
+  gtk_widget_set_sensitive(button, FALSE);
 
   gui_dialog_set_default_response(preport->shell, GTK_RESPONSE_CLOSE);
   gui_dialog_response_set_callback(preport->shell,
@@ -1713,7 +1750,7 @@ enum endgame_report_columns {
   FRD_COL_NUM
 };
 
-static struct endgame_report endgame_report = { NULL, };
+static struct endgame_report endgame_report = { NULL, NULL };
 
 /****************************************************************************
   Returns the title of the column (translated).
@@ -1758,7 +1795,7 @@ static void endgame_report_update(struct endgame_report *preport,
   col_types[FRD_COL_PLAYER] = G_TYPE_STRING;
   col_types[FRD_COL_NATION] = GDK_TYPE_PIXBUF;
   col_types[FRD_COL_SCORE] = G_TYPE_INT;
-  for (i = FRD_COL_NUM; i < col_num; i++) {
+  for (i = FRD_COL_NUM; (guint)i < col_num; i++) {
     col_types[i] = G_TYPE_INT;
   }
   store = gtk_list_store_newv(col_num, col_types);
@@ -1766,7 +1803,7 @@ static void endgame_report_update(struct endgame_report *preport,
   g_object_unref(G_OBJECT(store));
 
   /* Create the new columns. */
-  for (i = 0; i < col_num; i++) {
+  for (i = 0; (guint)i < col_num; i++) {
     GtkCellRenderer *renderer;
     const char *title;
     const char *attribute;
@@ -1848,13 +1885,18 @@ static void endgame_report_init(struct endgame_report *preport)
                                       GTK_SHADOW_ETCHED_IN);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
-  gtk_box_pack_start(GTK_BOX(preport->shell->vbox), sw, TRUE, TRUE, 0);
+  gtk_container_add(GTK_CONTAINER(preport->shell->vbox), sw);
 
   view = gtk_tree_view_new();
   gtk_widget_set_name(view, "small_font");
   gtk_container_add(GTK_CONTAINER(sw), view);
   preport->tree_view = GTK_TREE_VIEW(view);
 
+  if (preport->shell->type == GUI_DIALOG_TAB) {
+    gtk_widget_set_hexpand(GTK_WIDGET(view), TRUE);
+    gtk_widget_set_vexpand(GTK_WIDGET(view), TRUE);
+  }
+
   gui_dialog_show_all(preport->shell);
 }
 
