@@ -22,6 +22,7 @@
 
 /* common */
 #include "game.h"
+#include "movement.h"
 #include "unitlist.h"
 
 /* client */
@@ -685,7 +686,7 @@ void popup_diplomat_dialog(struct unit *pUnit, struct tile *ptile)
     }
       
     /* ---------- */
-    if (diplomat_can_do_action(pUnit, DIPLOMAT_MOVE, ptile)) {
+    if (unit_can_move_to_tile(pUnit, ptile, FALSE)) {
     
       create_active_iconlabel(pBuf, pWindow->dst, pStr,
 	    _("Keep moving"), diplomat_keep_moving_callback);
@@ -827,11 +828,11 @@ static int sabotage_impr_callback(struct widget *pWidget)
   return -1;
 }
 
-/****************************************************************
+/*************************************************************************
  Pops-up the Spy sabotage dialog, upon return of list of
  available improvements requested by the above function.
-*****************************************************************/
-void popup_sabotage_dialog(struct city *pCity)
+**************************************************************************/
+void popup_sabotage_dialog(struct unit *actor, struct city *pCity)
 {
   struct widget *pWindow = NULL, *pBuf = NULL , *pLast = NULL;
   struct CONTAINER *pCont;
@@ -898,6 +899,9 @@ void popup_sabotage_dialog(struct city *pCity)
     
   add_to_gui_list(ID_SEPARATOR, pBuf);
   area.h += pBuf->next->size.h;
+
+  pDiplomat_Dlg->pdialog->pEndActiveWidgetList = pBuf;
+
   /* ------------------ */
   n = 0;
   city_built_iterate(pCity, pImprove) {
@@ -913,11 +917,6 @@ void popup_sabotage_dialog(struct city *pCity)
     
       area.w = MAX(area.w , pBuf->size.w);
       imp_h += pBuf->size.h;
-      
-      if (!pDiplomat_Dlg->pdialog->pEndActiveWidgetList)
-      {
-	pDiplomat_Dlg->pdialog->pEndActiveWidgetList = pBuf;
-      }
     
       if (n > 9) {
         set_wflag(pBuf, WF_HIDDEN);
@@ -1121,7 +1120,7 @@ void popdown_incite_dialog(void)
   Popup a window asking a diplomatic unit if it wishes to incite the
   given enemy city.
 **************************************************************************/
-void popup_incite_dialog(struct city *pCity, int cost)
+void popup_incite_dialog(struct unit *actor, struct city *pCity, int cost)
 {
   struct widget *pWindow = NULL, *pBuf = NULL;
   SDL_String16 *pStr;
@@ -1372,7 +1371,7 @@ void popdown_bribe_dialog(void)
   Popup a dialog asking a diplomatic unit if it wishes to bribe the
   given enemy unit.
 **************************************************************************/
-void popup_bribe_dialog(struct unit *pUnit, int cost)
+void popup_bribe_dialog(struct unit *actor, struct unit *pUnit, int cost)
 {
   struct widget *pWindow = NULL, *pBuf = NULL;
   SDL_String16 *pStr;
