@@ -34,6 +34,7 @@
 /* common */
 #include "game.h"
 #include "improvement.h"
+#include "movement.h"
 #include "tech.h"
 #include "unitlist.h"
 
@@ -108,7 +109,7 @@ static void diplomat_bribe_callback(Widget w, XtPointer client_data,
 /**************************************************************************
   Creates and popups the bribe dialog
 **************************************************************************/
-void popup_bribe_dialog(struct unit *punit, int cost)
+void popup_bribe_dialog(struct unit *actor, struct unit *punit, int cost)
 {
   char tbuf[128], buf[128];
 
@@ -590,7 +591,7 @@ static void spy_request_sabotage_list(Widget w, XtPointer client_data,
   Pops-up the Spy sabotage dialog, upon return of list of
   available improvements requested by the above function.
 **************************************************************************/
-void popup_sabotage_dialog(struct city *pcity)
+void popup_sabotage_dialog(struct unit *actor, struct city *pcity)
 {  
   if(!spy_sabotage_shell){
     Position x, y;
@@ -654,7 +655,7 @@ static void diplomat_incite_callback(Widget w, XtPointer client_data,
 /**************************************************************************
   Popup the yes/no dialog for inciting, since we know the cost now
 **************************************************************************/
-void popup_incite_dialog(struct city *pcity, int cost)
+void popup_incite_dialog(struct unit *actor, struct city *pcity, int cost)
 {
   char tbuf[128], buf[128];
 
@@ -766,8 +767,9 @@ void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
 	XtSetSensitive(XtNameToWidget(diplomat_dialog, "*button3"), FALSE);
       if(!diplomat_can_do_action(punit, DIPLOMAT_INCITE, dest_tile))
 	XtSetSensitive(XtNameToWidget(diplomat_dialog, "*button4"), FALSE);
-      if(!diplomat_can_do_action(punit, DIPLOMAT_MOVE, dest_tile))
+      if (!unit_can_move_to_tile(punit, dest_tile, FALSE)) {
 	XtSetSensitive(XtNameToWidget(diplomat_dialog, "*button5"), FALSE);
+      }
     } else {
       diplomat_dialog =
         popup_message_dialog(toplevel, "spydialog", astr_str(&text),
@@ -793,8 +795,9 @@ void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
 	XtSetSensitive(XtNameToWidget(diplomat_dialog, "*button4"), FALSE);
       if(!diplomat_can_do_action(punit, DIPLOMAT_INCITE, dest_tile))
 	XtSetSensitive(XtNameToWidget(diplomat_dialog, "*button5"), FALSE);
-      if(!diplomat_can_do_action(punit, DIPLOMAT_MOVE, dest_tile))
+      if (!unit_can_move_to_tile(punit, dest_tile, FALSE)) {
 	XtSetSensitive(XtNameToWidget(diplomat_dialog, "*button6"), FALSE);
+      }
     }
   } else { 
     if ((ptunit = unit_list_get(dest_tile->units, 0))) {
