@@ -47,9 +47,9 @@ void caravan_dialog_update(void);
 void popup_diplomat_dialog(struct unit *punit, struct tile *ptile);
 int diplomat_handled_in_diplomat_dialog(void);
 void close_diplomat_dialog(void);
-void popup_incite_dialog(struct city *pcity, int cost);
-void popup_bribe_dialog(struct unit *punit, int cost);
-void popup_sabotage_dialog(struct city *pcity);
+void popup_incite_dialog(struct unit *actor, struct city *pcity, int cost);
+void popup_bribe_dialog(struct unit *actor, struct unit *punit, int cost);
+void popup_sabotage_dialog(struct unit *actor, struct city *pcity);
 void popup_pillage_dialog(struct unit *punit, bv_special may_pillage,
                           bv_bases bases);
 void popup_upgrade_dialog(struct unit_list *punits);
