@@ -164,7 +164,7 @@ const char *popup_info_text(struct tile *ptile)
     astr_add(&str, _("Unknown"));
     return astr_str(&str);
   }
-  astr_add_line(&str, _("Terrain: %s"),  tile_get_info_text(ptile, 0));
+  astr_add_line(&str, _("Terrain: %s"),  tile_get_info_text(ptile, TRUE, 0));
   astr_add_line(&str, _("Food/Prod/Trade: %s"),
 		get_tile_output_text(ptile));
   if (tile_has_special(ptile, S_HUT)) {
@@ -693,18 +693,18 @@ static int get_bulbs_per_turn(int *pours, bool *pteam, int *ptheirs)
   players_iterate(pplayer) {
     if (pplayer == client_player()) {
       city_list_iterate(pplayer->cities, pcity) {
-        ours += pcity->prod[O_SCIENCE];
+        ours += pcity->surplus[O_SCIENCE];
       } city_list_iterate_end;
 
       if (game.info.tech_upkeep_style == 1) {
-        ours -= player_research_get(pplayer)->tech_upkeep;
+        ours -= pplayer->client.tech_upkeep;
       }
     } else if (presearch == player_research_get(pplayer)) {
       team = TRUE;
-      theirs += pplayer->bulbs_last_turn;
+      theirs += pplayer->client.bulbs_prod;
 
       if (game.info.tech_upkeep_style == 1) {
-        theirs -= presearch->tech_upkeep;
+        theirs -= pplayer->client.tech_upkeep;;
       }
     }
   } players_iterate_end;
@@ -737,18 +737,19 @@ const char *science_dialog_text(void)
   perturn = get_bulbs_per_turn(&ours, &team, &theirs);
 
   research = player_research_get(client_player());
-  upkeep = research->tech_upkeep;
+  upkeep = client_player()->client.tech_upkeep;
 
   if (NULL == client.conn.playing || (ours == 0 && theirs == 0
                                       && upkeep == 0)) {
     return _("Progress: no research");
   }
 
-  if (A_UNSET == research->researching) {
+  if (!is_future_tech(research->researching)
+      && NULL == valid_advance_by_number(research->researching)) {
     astr_add(&str, _("Progress: no research"));
   } else {
     int done = research->bulbs_researched;
-    int total = total_bulbs_required(client_player());
+    int total = research->client.researching_cost;
 
     if (perturn > 0) {
       int turns = MAX(1, ceil((double)total) / perturn);
@@ -811,13 +812,14 @@ const char *get_science_target_text(double *percent)
   }
 
   astr_clear(&str);
-  if (research->researching == A_UNSET) {
+  if (!is_future_tech(research->researching)
+      && NULL == valid_advance_by_number(research->researching)) {
     astr_add(&str, _("%d/- (never)"), research->bulbs_researched);
     if (percent) {
       *percent = 0.0;
     }
   } else {
-    int total = total_bulbs_required(client.conn.playing);
+    int total = research->client.researching_cost;
     int done = research->bulbs_researched;
     int perturn = get_bulbs_per_turn(NULL, NULL, NULL);
 
@@ -963,7 +965,7 @@ const char *get_info_label_text_popup(void)
 
   if (NULL != client.conn.playing) {
     int perturn = get_bulbs_per_turn(NULL, NULL, NULL);
-    int upkeep = player_research_get(client_player())->tech_upkeep;
+    int upkeep = client_player()->client.tech_upkeep;
 
     astr_add_line(&str, _("Gold: %d"),
 		  client.conn.playing->economic.gold);
@@ -1092,7 +1094,7 @@ const char *get_unit_info_label_text2(struct unit_list *punits, int linebreaks)
     struct city *pcity = player_city_by_number(unit_owner(punit),
                                                punit->homecity);
 
-    astr_add_line(&str, "%s", tile_get_info_text(unit_tile(punit),
+    astr_add_line(&str, "%s", tile_get_info_text(unit_tile(punit), TRUE,
                                                  linebreaks));
     {
       const char *infratext
@@ -1312,13 +1314,14 @@ const char *get_bulb_tooltip(void)
   if (NULL != client.conn.playing) {
     struct player_research *research = player_research_get(client_player());
 
-    if (research->researching == A_UNSET) {
+    if (!is_future_tech(research->researching)
+        && NULL == valid_advance_by_number(research->researching)) {
       astr_add_line(&str, _("no research target."));
     } else {
       int turns = 0;
       int perturn = get_bulbs_per_turn(NULL, NULL, NULL);
       int done = research->bulbs_researched;
-      int total = total_bulbs_required(client_player());
+      int total = research->client.researching_cost;
       struct astring buf1 = ASTRING_INIT, buf2 = ASTRING_INIT;
 
       if (perturn > 0) {
@@ -1340,7 +1343,7 @@ const char *get_bulb_tooltip(void)
       astr_add_line(&str, _("%s: %d/%d (%s, %s)."),
                     advance_name_researching(client.conn.playing),
                     research->bulbs_researched,
-                    total_bulbs_required(client.conn.playing),
+                    research->client.researching_cost,
                     astr_str(&buf1), astr_str(&buf2));
       
       astr_free(&buf1);
