@@ -363,14 +363,16 @@ struct packet_player_info {
   int tax;
   int science;
   int luxury;
-  int bulbs_last_turn_old;
-  int bulbs_last_turn_new;
+  int bulbs_last_turn;
+  int bulbs_prod;
   int bulbs_researched;
   int techs_researched;
   int researching;
+  int researching_cost;
   int science_cost;
   int future_tech;
   int tech_goal;
+  int tech_upkeep;
   bool is_connected;
   int revolution_finishes;
   bool ai;
