@@ -871,9 +871,13 @@ static enum fc_tristate is_terrain_class_in_range(const struct tile *target_tile
     /* The requirement is filled if the tile has the terrain of correct class. */
     return BOOL_TO_TRISTATE(terrain_belongs_to_class(tile_terrain(target_tile), class));
   case REQ_RANGE_CADJACENT:
-    return BOOL_TO_TRISTATE(is_terrain_class_card_near(target_tile, class));
+    return BOOL_TO_TRISTATE(terrain_belongs_to_class(tile_terrain(target_tile),
+                                                     class)
+                            || is_terrain_class_card_near(target_tile, class));
   case REQ_RANGE_ADJACENT:
-    return BOOL_TO_TRISTATE(is_terrain_class_near_tile(target_tile, class));
+    return BOOL_TO_TRISTATE(terrain_belongs_to_class(tile_terrain(target_tile),
+                                                     class)
+                            || is_terrain_class_near_tile(target_tile, class));
   case REQ_RANGE_CITY:
   case REQ_RANGE_CONTINENT:
   case REQ_RANGE_PLAYER:
@@ -903,9 +907,11 @@ static enum fc_tristate is_base_type_in_range(const struct tile *target_tile,
     /* The requirement is filled if the tile has base of requested type. */
     return BOOL_TO_TRISTATE(tile_has_base(target_tile, pbase));
   case REQ_RANGE_CADJACENT:
-    return BOOL_TO_TRISTATE(is_base_card_near(target_tile, pbase));
+    return BOOL_TO_TRISTATE(tile_has_base(target_tile, pbase)
+                            || is_base_card_near(target_tile, pbase));
   case REQ_RANGE_ADJACENT:
-    return BOOL_TO_TRISTATE(is_base_near_tile(target_tile, pbase));
+    return BOOL_TO_TRISTATE(tile_has_base(target_tile, pbase)
+                            || is_base_near_tile(target_tile, pbase));
   case REQ_RANGE_CITY:
   case REQ_RANGE_CONTINENT:
   case REQ_RANGE_PLAYER:
@@ -1077,6 +1083,9 @@ static enum fc_tristate is_citytile_in_range(const struct tile *target_tile,
       case REQ_RANGE_LOCAL:
         return BOOL_TO_TRISTATE(is_city_in_tile(target_tile, target_city));
       case REQ_RANGE_CADJACENT:
+        if (is_city_in_tile(target_tile, target_city)) {
+          return TRI_YES;
+        }
         cardinal_adjc_iterate(target_tile, adjc_tile) {
           if (is_city_in_tile(adjc_tile, target_city)) {
             return TRI_YES;
@@ -1085,6 +1094,9 @@ static enum fc_tristate is_citytile_in_range(const struct tile *target_tile,
 
         return TRI_NO;
       case REQ_RANGE_ADJACENT:
+        if (is_city_in_tile(target_tile, target_city)) {
+          return TRI_YES;
+        }
         adjc_iterate(target_tile, adjc_tile) {
           if (is_city_in_tile(adjc_tile, target_city)) {
             return TRI_YES;
@@ -1538,16 +1550,22 @@ const char *universal_name_translation(const struct universal *psource,
     fc_strlcat(buf, utype_name_translation(psource->value.utype), bufsz);
     return buf;
   case VUT_UTFLAG:
-    cat_snprintf(buf, bufsz, _("\"%s\" units"),
-		 /* flag names are never translated */
+    cat_snprintf(buf, bufsz,
+                 /* TRANS: Unit type flag */
+                 Q_("?utflag:\"%s\" units"),
+                 /* flag names are never translated */
 		 unit_flag_rule_name(psource->value.unitflag));
     return buf;
   case VUT_UCLASS:
-    cat_snprintf(buf, bufsz, _("%s units"),
+    cat_snprintf(buf, bufsz,
+                 /* TRANS: Unit class */
+                 _("%s units"),
 		 uclass_name_translation(psource->value.uclass));
     return buf;
   case VUT_UCFLAG:
-    cat_snprintf(buf, bufsz, _("\"%s\" units"),
+    cat_snprintf(buf, bufsz,
+                 /* TRANS: Unit class flag */
+                 Q_("?ucflag:\"%s\" units"),
                  /* flag names are never translated */
                  unit_class_flag_id_name(psource->value.unitclassflag));
     return buf;
@@ -1569,14 +1587,12 @@ const char *universal_name_translation(const struct universal *psource,
                  ai_level_name(psource->value.ai_level)); /* FIXME */
     return buf;
   case VUT_TERRAINCLASS:
-    /* TRANS: "Land terrain" */
+    /* TRANS: Terrain class: "Land terrain" */
     cat_snprintf(buf, bufsz, _("%s terrain"),
                  terrain_class_name_translation(psource->value.terrainclass));
     return buf;
   case VUT_BASE:
-    /* TRANS: "Fortress base" */
-    cat_snprintf(buf, bufsz, _("%s base"),
-                 base_name_translation(psource->value.base));
+    fc_strlcat(buf, base_name_translation(psource->value.base), bufsz);
     return buf;
   case VUT_MINYEAR:
     cat_snprintf(buf, bufsz, _("After %s"),
@@ -1588,7 +1604,7 @@ const char *universal_name_translation(const struct universal *psource,
                  terrain_alteration_name_translation(psource->value.terrainalter));
     return buf;
   case VUT_CITYTILE:
-    fc_strlcat(buf, _("City center tile"), bufsz);
+    fc_strlcat(buf, _("City center"), bufsz);
     return buf;
   case VUT_COUNT:
     break;
