@@ -48,8 +48,6 @@ struct advance advances[A_LAST];
  * the sciencebox multiplier. */
 static double techcoststyle1[A_LAST];
 
-static int tech_upkeep_calc(const struct player *pplayer);
-
 /**************************************************************************
   Return the last item of advances/technologies.
 **************************************************************************/
@@ -293,41 +291,57 @@ bool player_invention_reachable(const struct player *pplayer,
                                 const Tech_type_id tech,
                                 bool allow_prereqs)
 {
-  Tech_type_id root;
-
-  if (!valid_advance_by_number(tech)) {
+  if (valid_advance_by_number(tech) == NULL) {
     return FALSE;
-  }
+  } else if (advance_required(tech, AR_ROOT) != A_NONE) {
+    /* 'tech' has at least one root requirement. We need to check them
+     * all. */
+    bv_techs done;
+    Tech_type_id techs[game.control.num_tech_types];
+    Tech_type_id root;
+    enum tech_req req;
+    int techs_num;
+    int i;
 
-  root = advance_required(tech, AR_ROOT);
-  if (A_NONE != root) {
-    if (root == tech) {
-      /* This tech requires itself; it can only be reached by special means
-       * (init_techs, lua script, ...).
-       * If you already know it, you can "reach" it; if not, not. (This case
-       * is needed for descendants of this tech.) */
-      return TECH_KNOWN == player_invention_state(pplayer, tech);
-    } else if (allow_prereqs) {
-      /* Recursive check if the player can ever reach this tech (root tech
-       * and both requirements). */
-      return (player_invention_reachable(pplayer, root, TRUE)
-              && player_invention_reachable(pplayer,
-                                            advance_required(tech, AR_ONE),
-                                            allow_prereqs)
-              && player_invention_reachable(pplayer,
-                                            advance_required(tech, AR_TWO),
-                                            allow_prereqs));
-    } else if (TECH_KNOWN != player_invention_state(pplayer, root)
-               || !player_invention_reachable(pplayer,
-                                              advance_required(tech, AR_ONE),
-                                              allow_prereqs)
-               || !player_invention_reachable(pplayer,
-                                              advance_required(tech, AR_TWO),
-                                              allow_prereqs)) {
-      /* This tech requires knowledge of another tech (root tech or recursive
-       * a root tech of a requirement) before being available. Prevents
-       * sharing of untransferable techs. */
-      return FALSE;
+    techs[0] = tech;
+    BV_CLR_ALL(done);
+    BV_SET(done, A_NONE);
+    BV_SET(done, tech);
+    techs_num = 1;
+
+    for (i = 0; i < techs_num; i++) {
+      root = advance_required(techs[i], AR_ROOT);
+      if (root == techs[i]) {
+        /* This tech requires itself; it can only be reached by special
+         * means (init_techs, lua script, ...).
+         * If you already know it, you can "reach" it; if not, not. (This
+         * case is needed for descendants of this tech.) */
+        if (player_invention_state(pplayer, root) != TECH_KNOWN) {
+          return FALSE;
+        }
+      } else if (!allow_prereqs
+                 && player_invention_state(pplayer, root) != TECH_KNOWN) {
+        /* This tech requires knowledge of another tech (root tech) before
+         * being available. Prevents sharing of untransferable techs. */
+        return FALSE;
+      } else {
+        /* Check if requirements are reachable. */
+        Tech_type_id req_tech;
+
+        for (req = 0; req < (allow_prereqs ? AR_SIZE : AR_ROOT); req++) {
+          req_tech = advance_required(techs[i], req);
+          if (!valid_advance_by_number(req_tech)) {
+            return FALSE;
+          } else if (!BV_ISSET(done, req_tech)) {
+            if (advance_required(req_tech, AR_ROOT) != A_NONE) {
+              fc_assert(techs_num < ARRAY_SIZE(techs));
+              techs[techs_num] = req_tech;
+              techs_num++;
+            }
+            BV_SET(done, req_tech);
+          }
+        }
+      }
     }
   }
 
@@ -408,78 +422,98 @@ void player_research_update(struct player *pplayer)
       }
     } advance_index_iterate_end;
   }
-
-  /* calculate tech upkeep cost */
-  if (game.info.tech_upkeep_style == 1) {
-    /* upkeep activated in the ruleset */
-    research->tech_upkeep = tech_upkeep_calc(pplayer);
-
-    log_debug("[%s (%d)] tech upkeep: %d", player_name(pplayer),
-              player_index(pplayer), research->tech_upkeep);
-  } else {
-    /* upkeep deactivated in the ruleset */
-    research->tech_upkeep = 0;
-  }
 }
 
 /**************************************************************************
   Calculate the bulb upkeep needed for all techs of a player. See also
   base_total_bulbs_required().
 **************************************************************************/
-static int tech_upkeep_calc(const struct player *pplayer)
+int player_tech_upkeep(const struct player *pplayer)
 {
-  struct player_research *research = player_research_get(pplayer);
-  int tech_cost_style = game.info.tech_cost_style;
-  int f = research->future_tech, t = research->techs_researched;
-  double tech_bulb_sum = 0.0;
+  const struct player_research *presearch = player_research_get(pplayer);
+  int f = presearch->future_tech, t = presearch->techs_researched;
+  double tech_upkeep = 0.0;
+  double total_research_factor;
+  int members;
 
-  if (!pplayer) {
+  if (0 == game.info.tech_upkeep_style) {
     return 0;
   }
 
-  /* upkeep cost for 'normal' techs (t) */
-  switch (tech_cost_style) {
+  total_research_factor = 0.0;
+  members = 0;
+  players_iterate_alive(member) {
+    if (player_research_get(member) == presearch) {
+      total_research_factor += (get_player_bonus(member, EFT_TECH_COST_FACTOR)
+                                + (member->ai_controlled
+                                   ? member->ai_common.science_cost / 100.0
+                                   : 1));
+      members++;
+    }
+  } players_iterate_alive_end;
+  if (0 == members) {
+    /* No player still alive. */
+    return 0;
+  }
+
+  /* Upkeep cost for 'normal' techs (t). */
+  switch (game.info.tech_cost_style) {
   case 0:
     /* sum_1^t x = t * (t + 1) / 2 */
-    tech_bulb_sum += (double)t * (t + 1) / 2 * game.info.base_tech_cost;
+    tech_upkeep += game.info.base_tech_cost * t * (t + 1) / 2;
     break;
   case 1:
   case 3:
-    advance_index_iterate(A_NONE, i) {
-      if (player_invention_state(pplayer, i) == TECH_KNOWN) {
-        tech_bulb_sum += techcoststyle1[i];
+    advance_index_iterate(A_FIRST, i) {
+      if (TECH_KNOWN == player_invention_state(pplayer, i)) {
+        tech_upkeep += techcoststyle1[i];
       }
     } advance_index_iterate_end;
+    if (0 < f) {
+      /* Upkeep cost for future techs (f) are calculated using style 0:
+       * sum_t^(t+f) x = (f * (2 * t + f + 1) + 2 * t) / 2 */
+      tech_upkeep += (double) (game.info.base_tech_cost
+                               * (f * (2 * t + f + 1) + 2 * t) / 2);
+    }
     break;
   case 2:
   case 4:
-    advance_index_iterate(A_NONE, i) {
-      if (player_invention_state(pplayer, i) == TECH_KNOWN) {
+    advance_index_iterate(A_FIRST, i) {
+      if (TECH_KNOWN == player_invention_state(pplayer, i)) {
         if (advances[i].preset_cost != -1) {
-          tech_bulb_sum += advances[i].preset_cost;
+          tech_upkeep += advances[i].preset_cost;
         } else {
-          tech_bulb_sum += techcoststyle1[i];
+          tech_upkeep += techcoststyle1[i];
         }
       }
     } advance_index_iterate_end;
+    if (0 < f) {
+      /* Upkeep cost for future techs (f) are calculated using style 0:
+       * sum_t^(t+f) x = (f * (2 * t + f + 1) + 2 * t) / 2 */
+      tech_upkeep += (double) (game.info.base_tech_cost
+                               * (f * (2 * t + f + 1) + 2 * t) / 2);
+    }
     break;
   default:
-    fc_assert_ret_val_msg(FALSE, 0, "Invalid tech_cost_style %d %d",
-                          game.info.tech_cost_style,
-                          tech_cost_style);
+    fc_assert_msg(FALSE, "Invalid tech_cost_style %d",
+                  game.info.tech_cost_style);
   }
 
-  /* upkeep cost for future techs (f) are calculated using style 0:
-   * sum_t^(t+f) x = (f * (2 * t + f + 1) + 2 * t) / 2 */
-  tech_bulb_sum += (double)(f * (2 * t + f + 1) + 2 * t) / 2
-                           * game.info.base_tech_cost;
+  tech_upkeep *= total_research_factor / members;
+  tech_upkeep *= (double) game.info.sciencebox / 100.0;
+  /* We only want to calculate the upkeep part of one player, not the
+   * whole team! */
+  tech_upkeep /= members;
+  tech_upkeep /= game.info.tech_upkeep_divider;
+  tech_upkeep -= get_player_bonus(pplayer, EFT_TECH_UPKEEP_FREE);
 
-  tech_bulb_sum *= get_player_bonus(pplayer, EFT_TECH_COST_FACTOR);
-  tech_bulb_sum *= (double)game.info.sciencebox / 100.0;
-  tech_bulb_sum /= game.info.tech_upkeep_divider;
-  tech_bulb_sum -= get_player_bonus(pplayer, EFT_TECH_UPKEEP_FREE);
+  if (0.0 > tech_upkeep) {
+    tech_upkeep = 0.0;
+  }
 
-  return MAX((int)tech_bulb_sum, 0);
+  log_debug("[%s (%d)] tech upkeep: %d", player_name(pplayer),
+            player_number(pplayer), (int) tech_upkeep);
+  return (int) tech_upkeep;
 }
 
 /**************************************************************************
@@ -491,7 +525,7 @@ Tech_type_id player_research_step(const struct player *pplayer,
 {
   Tech_type_id sub_goal;
 
-  if (!player_invention_reachable(pplayer, goal, FALSE)) {
+  if (!player_invention_reachable(pplayer, goal, TRUE)) {
     return A_UNSET;
   }
   switch (player_invention_state(pplayer, goal)) {
@@ -643,8 +677,17 @@ int total_bulbs_required(const struct player *pplayer)
 int base_total_bulbs_required(const struct player *pplayer,
 			      Tech_type_id tech)
 {
+  const struct player_research *presearch = (pplayer != NULL
+                                             ? player_research_get(pplayer)
+                                             : NULL);
   int tech_cost_style = game.info.tech_cost_style;
-  double base_cost;
+  int members;
+  double base_cost, total_cost;
+
+  if (A_UNSET == tech
+      || A_UNKNOWN == tech) {
+    return 0;
+  }
 
   if (pplayer
       && !is_future_tech(tech)
@@ -670,9 +713,8 @@ int base_total_bulbs_required(const struct player *pplayer,
 
   switch (tech_cost_style) {
   case 0:
-    if (pplayer) {
-      base_cost = player_research_get(pplayer)->techs_researched 
-	* game.info.base_tech_cost;
+    if (presearch != NULL) {
+      base_cost = presearch->techs_researched * game.info.base_tech_cost;
     } else {
       base_cost = 0;
     }
@@ -691,7 +733,21 @@ int base_total_bulbs_required(const struct player *pplayer,
     base_cost = 0.0;
   }
 
-  base_cost *= get_player_bonus(pplayer, EFT_TECH_COST_FACTOR);
+  total_cost = 0.0;
+  members = 0;
+  players_iterate_alive(member) {
+    if (player_research_get(member) == presearch) {
+      members++;
+      total_cost += (base_cost
+                     * get_player_bonus(member, EFT_TECH_COST_FACTOR));
+    }
+  } players_iterate_alive_end;
+  if (0 == members) {
+    /* There is no more alive players for this research, no need to apply
+     * complicated modifiers. */
+    return base_cost * (double) game.info.sciencebox / 100.0;
+  }
+  base_cost = total_cost / members;
 
   switch (game.info.tech_leakage) {
   case 0:
@@ -702,16 +758,31 @@ int base_total_bulbs_required(const struct player *pplayer,
     {
       int players = 0, players_with_tech_and_embassy = 0;
 
-      players_iterate_alive(other) {
-	players++;
-	if (player_invention_state(other, tech) == TECH_KNOWN
-	    && pplayer && player_has_embassy(pplayer, other)) {
-	  players_with_tech_and_embassy++;
-	}
+      players_iterate_alive(aplayer) {
+        const struct player_research *aresearch =
+            player_research_get(aplayer);
+
+        players++;
+        if (aresearch == presearch
+            || (A_FUTURE == tech
+                ? aresearch->future_tech <= presearch->future_tech
+                : TECH_KNOWN != player_invention_state(aplayer, tech))) {
+          continue;
+        }
+
+        players_iterate_alive(member) {
+          if (player_research_get(member) == presearch
+              && player_has_embassy(member, aplayer)) {
+            players_with_tech_and_embassy++;
+            break;
+          }
+        } players_iterate_alive_end;
       } players_iterate_alive_end;
 
-      base_cost *= (double)(players - players_with_tech_and_embassy);
-      base_cost /= (double)players;
+      fc_assert_ret_val(0 < players, base_cost);
+      fc_assert(players >= players_with_tech_and_embassy);
+      base_cost *= (double) (players - players_with_tech_and_embassy);
+      base_cost /= (double) players;
     }
     break;
 
@@ -719,15 +790,20 @@ int base_total_bulbs_required(const struct player *pplayer,
     {
       int players = 0, players_with_tech = 0;
 
-      players_iterate_alive(other) {
-	players++;
-	if (player_invention_state(other, tech) == TECH_KNOWN) {
-	  players_with_tech++;
-	}
+      players_iterate_alive(aplayer) {
+        players++;
+        if (A_FUTURE == tech
+            ? (player_research_get(aplayer)->future_tech
+               > presearch->future_tech)
+            : TECH_KNOWN == player_invention_state(aplayer, tech)) {
+          players_with_tech++;
+        }
       } players_iterate_alive_end;
 
-      base_cost *= (double)(players - players_with_tech);
-      base_cost /= (double)players;
+      fc_assert_ret_val(0 < players, base_cost);
+      fc_assert(players >= players_with_tech);
+      base_cost *= (double) (players - players_with_tech);
+      base_cost /= (double) players;
     }
     break;
 
@@ -735,18 +811,23 @@ int base_total_bulbs_required(const struct player *pplayer,
     {
       int players = 0, players_with_tech = 0;
 
-      players_iterate_alive(other) {
-	if (is_barbarian(other)) {
-	  continue;
-	}
-	players++;
-	if (player_invention_state(other, tech) == TECH_KNOWN) {
-	  players_with_tech++;
-	}
+      players_iterate_alive(aplayer) {
+        if (is_barbarian(aplayer)) {
+          continue;
+        }
+        players++;
+        if (A_FUTURE == tech
+            ? (player_research_get(aplayer)->future_tech
+               > presearch->future_tech)
+            : TECH_KNOWN == player_invention_state(aplayer, tech)) {
+          players_with_tech++;
+        }
       } players_iterate_alive_end;
 
-      base_cost *= (double)(players - players_with_tech);
-      base_cost /= (double)players;
+      fc_assert_ret_val(0 < players, base_cost);
+      fc_assert(players >= players_with_tech);
+      base_cost *= (double) (players - players_with_tech);
+      base_cost /= (double) players;
     }
     break;
 
@@ -758,12 +839,21 @@ int base_total_bulbs_required(const struct player *pplayer,
    * can also be adopted to create an extra-hard AI skill level where the AI
    * gets science benefits */
 
-  if (pplayer && pplayer->ai_controlled) {
-    fc_assert_ret_val(pplayer->ai_common.science_cost > 0, FC_INFINITY);
-    base_cost *= (double)pplayer->ai_common.science_cost / 100.0;
-  }
+  total_cost = 0.0;
+  players_iterate_alive(member) {
+    if (player_research_get(member) != presearch) {
+      continue;
+    }
+    if (member->ai_controlled) {
+      fc_assert(0 < member->ai_common.science_cost);
+      total_cost += base_cost * member->ai_common.science_cost / 100.0;
+    } else {
+      total_cost += base_cost;
+    }
+  } players_iterate_alive_end;
+  base_cost = total_cost / members;
 
-  base_cost *= (double)game.info.sciencebox / 100.0;
+  base_cost *= (double) game.info.sciencebox / 100.0;
 
   return MAX(base_cost, 1);
 }
